@@ -1,5 +1,5 @@
 ;;; dgnushack.el --- a hack to set the load path for byte-compiling
-;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002
+;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003
 ;;        Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
@@ -49,8 +49,12 @@
 (defalias 'facep 'ignore)
 
 (require 'cl)
-(unless (dolist (var nil t))
-  ;; Override the macro `dolist' which may have been defined in egg.el.
+
+(unless (and
+	 ;; `dolist' might not be available because of ``-no-autoloads''.
+	 (fboundp 'dolist)
+	 ;; It may have been defined in egg.el.
+	 (dolist (var nil t)))
   (load "cl-macs" nil t))
 
 (defvar srcdir (or (getenv "srcdir") "."))
@@ -95,6 +99,43 @@
 ;  (cons 'progn (cdr form)))
 ;(defalias 'byte-compile-file-form-defsubst 'byte-compile-file-form-defun)
 
+(when (and (not (featurep 'xemacs))
+	   (= emacs-major-version 21)
+	   (= emacs-minor-version 3)
+	   (condition-case code
+	       (let ((byte-compile-error-on-warn t))
+		 (byte-optimize-form (quote (pop x)) t)
+		 nil)
+	     (error (string-match "called for effect"
+				  (error-message-string code)))))
+  (defadvice byte-optimize-form-code-walker (around silence-warn-for-pop
+						    (form for-effect)
+						    activate)
+    "Silence the warning \"...called for effect\" for the `pop' form.
+It is effective only when the `pop' macro is defined by cl.el rather
+than subr.el."
+    (let (tmp)
+      (if (and (eq (car-safe form) 'car)
+	       for-effect
+	       (setq tmp (get 'car 'side-effect-free))
+	       (not byte-compile-delete-errors)
+	       (not (eq tmp 'error-free))
+	       (eq (car-safe (cadr form)) 'prog1)
+	       (let ((var (cadr (cadr form)))
+		     (last (nth 2 (cadr form))))
+		 (and (symbolp var)
+		      (null (nthcdr 3 (cadr form)))
+		      (eq (car-safe last) 'setq)
+		      (eq (cadr last) var)
+		      (eq (car-safe (nth 2 last)) 'cdr)
+		      (eq (cadr (nth 2 last)) var))))
+	  (progn
+	    (put 'car 'side-effect-free 'error-free)
+	    (unwind-protect
+		ad-do-it
+	      (put 'car 'side-effect-free tmp)))
+	ad-do-it))))
+
 (when (boundp 'MULE)
   (let (current-load-list)
     ;; Make the function to be silent at compile-time.
@@ -131,7 +172,7 @@ to the specified name LIBRARY (a la calling `load' instead of `load-library')."
 	(byte-optimize-form
 	 '(and
 	   (< 0 1)
-	   (message "The subform `(< 0 1)' should be optimized to `t'"))
+	   (message "The subform `(< 0 1)' should be optimized to t"))
 	 'for-effect))
        '(< 0 1))
   (defadvice byte-optimize-form-code-walker
@@ -210,6 +251,7 @@ It has already been fixed in XEmacs since 1999-12-06."
 	  (unless (or (member parent load-path)
 		      (member (file-name-as-directory parent) load-path))
 	    (push parent (cdr load-path))))
+	(require 'advice)
 	(require 'path-util))
     (error "
 APEL modules are not found in %s.
@@ -347,7 +389,8 @@ Try to re-configure with --with-addpath=FLIM_PATH and run make again.
 				     (error nil)))
 		   (when (listp form)
 		     (while form
-		       (setq elem (pop form))
+		       (setq elem (car-safe form)
+			     form (cdr-safe form))
 		       (unless (memq (car-safe elem)
 				     '(defcustom defface defgroup
 				       define-widget quote))
@@ -460,11 +503,77 @@ Try to re-configure with --with-addpath=FLIM_PATH and run make again.
 (defalias 'ange-ftp-re-read-dir 'ignore)
 (defalias 'define-mail-user-agent 'ignore)
 
+(eval-and-compile
+  (when (featurep 'xemacs)
+    ;; XEmacs 21.1 needs some extra hand holding
+    (when (eq emacs-minor-version 1)
+      (autoload 'custom-declare-face "cus-face" nil t)
+      (autoload 'cl-compile-time-init "cl-macs" nil t)
+      (autoload 'defadvice "advice" nil nil 'macro))
+    (unless (fboundp 'defadvice)
+      (autoload 'defadvice "advice" nil nil 'macro))
+    (autoload 'Info-directory "info" nil t)
+    (autoload 'Info-menu "info" nil t)
+    (autoload 'annotations-at "annotations")
+    (autoload 'apropos "apropos" nil t)
+    (autoload 'apropos-command "apropos" nil t)
+    (autoload 'bbdb-complete-name "bbdb-com" nil t)
+    (autoload 'browse-url "browse-url" nil t)
+    (autoload 'customize-apropos "cus-edit" nil t)
+    (autoload 'customize-save-variable "cus-edit" nil t)
+    (autoload 'customize-variable "cus-edit" nil t)
+    (autoload 'delete-annotation "annotations")
+    (autoload 'dolist "cl-macs" nil nil 'macro)
+    (autoload 'enriched-decode "enriched")
+    (autoload 'info "info" nil t)
+    (autoload 'make-annotation "annotations")
+    (autoload 'make-display-table "disp-table")
+    (autoload 'pp "pp")
+    (autoload 'ps-despool "ps-print" nil t)
+    (autoload 'ps-spool-buffer "ps-print" nil t)
+    (autoload 'ps-spool-buffer-with-faces "ps-print" nil t)
+    (autoload 'read-passwd "passwd")
+    (autoload 'regexp-opt "regexp-opt")
+    (autoload 'reporter-submit-bug-report "reporter")
+    (if (emacs-version>= 21 5)
+	(autoload 'setenv "process" nil t)
+      (autoload 'setenv "env" nil t))
+    (autoload 'smtpmail-send-it "smtpmail")
+    (autoload 'sort-numeric-fields "sort" nil t)
+    (autoload 'sort-subr "sort")
+    (autoload 'trace-function-background "trace" nil t)
+    (autoload 'w3-do-setup "w3")
+    (autoload 'w3-prepare-buffer "w3-display")
+    (autoload 'w3-region "w3-display" nil t)
+    (defalias 'frame-char-height 'frame-height)
+    (defalias 'frame-char-width 'frame-width)
+    (defalias 'frame-parameter 'frame-property)
+    (defalias 'make-overlay 'ignore)
+    (defalias 'overlay-end 'ignore)
+    (defalias 'overlay-get 'ignore)
+    (defalias 'overlay-put 'ignore)
+    (defalias 'overlay-start 'ignore)
+    (defalias 'overlays-in 'ignore)
+    (defalias 'replace-dehighlight 'ignore)
+    (defalias 'replace-highlight 'ignore)
+    (defalias 'run-with-idle-timer 'ignore)
+    (defalias 'w3-coding-system-for-mime-charset 'ignore)))
+
+;; T-gnus stuff.
+(eval-and-compile
+  (when (featurep 'xemacs)
+    (autoload 'c-mode "cc-mode" nil t)
+    (autoload 'font-lock-mode "font-lock" nil t)
+    (autoload 'read-kbd-macro "edmacro" nil t)
+    (autoload 'turn-on-font-lock "font-lock" nil t))
+  (autoload 'nnheader-detect-coding-region "nnheader")
+  (autoload 'std11-extract-addresses-components "nnheader")
+  (autoload 'std11-fold-region "nnheader")
+  (autoload 'std11-narrow-to-field "nnheader")
+  (autoload 'std11-unfold-region "nnheader"))
+
 (defconst dgnushack-unexporting-files
   (append '("dgnushack.el" "dgnuspath.el" "dgnuskwds.el" "lpath.el")
-	  (condition-case nil
-	      (progn (require 'shimbun) nil)
-	    (error '("nnshimbun.el")))
 	  (unless (or (condition-case code
 			  (require 'w3-parse)
 			(error
@@ -530,7 +639,9 @@ Try to re-configure with --with-addpath=FLIM_PATH and run make again.
 	  (when (and (fboundp 'md5) (subrp (symbol-function 'md5)))
 	    '("md5.el"))
 	  (unless (boundp 'MULE)
-	    '("canlock-om.el")))
+	    '("canlock-om.el"))
+	  (when (featurep 'xemacs)
+	    '("gnus-load.el")))
   "Files which will not be installed.")
 
 (defconst dgnushack-exporting-files
@@ -544,8 +655,25 @@ Try to re-configure with --with-addpath=FLIM_PATH and run make again.
   "Print name of files which will be installed."
   (princ (mapconcat 'identity dgnushack-exporting-files " ")))
 
+(defconst dgnushack-dont-compile-files
+  '("gnus-load.el"
+    "mm-bodies.el" "mm-decode.el" "mm-encode.el" "mm-extern.el"
+    "mm-partial.el" "mm-url.el" "mm-uu.el" "mm-view.el" "mml-sec.el"
+    "mml-smime.el" "mml.el" "mml1991.el" "mml2015.el")
+  "Files which should not be byte-compiled.")
+
+(defun dgnushack-compile-verbosely ()
+  "Call dgnushack-compile with warnings ENABLED.  If you are compiling
+patches to gnus, you should consider modifying make.bat to call
+dgnushack-compile-verbosely.  All other users should continue to use
+dgnushack-compile."
+  (dgnushack-compile t))
+
 (defun dgnushack-compile (&optional warn)
   ;;(setq byte-compile-dynamic t)
+  (when (and (not (featurep 'xemacs))
+	     (< emacs-major-version 21))
+    (setq max-specpdl-size 1200))
   (unless warn
     (setq byte-compile-warnings
 	  '(free-vars unresolved callargs redefine)))
@@ -569,37 +697,54 @@ Modify to suit your needs."))
 	       (file-newer-than-file-p file elc))
       (delete-file elc)))
 
+  ;; Avoid barfing (from gnus-xmas) because the etc directory is not yet
+  ;; installed.
+  (when (featurep 'xemacs)
+    (setq gnus-xmas-glyph-directory "dummy"))
+
   (let ((files dgnushack-exporting-files)
 	;;(byte-compile-generate-call-tree t)
 	file elc)
-    ;; Avoid barfing (from gnus-xmas) because the etc directory is not yet
-    ;; installed.
-    (when (featurep 'xemacs)
-      (setq gnus-xmas-glyph-directory "dummy"))
     (while (setq file (pop files))
-      (setq file (expand-file-name file srcdir))
-      (when (or (not (file-exists-p
-		      (setq elc (concat (file-name-nondirectory file) "c"))))
-		(file-newer-than-file-p file elc))
-	(ignore-errors
-	  (byte-compile-file file))))))
+      (unless (member file dgnushack-dont-compile-files)
+	(setq file (expand-file-name file srcdir))
+	(when (or (not (file-exists-p
+			(setq elc (concat (file-name-nondirectory file) "c"))))
+		  (file-newer-than-file-p file elc))
+	  (ignore-errors
+	    (byte-compile-file file)))))))
 
 (defun dgnushack-recompile ()
   (require 'gnus)
   (byte-recompile-directory "." 0))
 
-(defvar dgnushack-gnus-load-file (expand-file-name "gnus-load.el" srcdir))
-(defvar dgnushack-cus-load-file (expand-file-name "cus-load.el" srcdir))
-(defvar dgnushack-auto-load-file (expand-file-name "auto-autoloads.el" srcdir))
+(defvar dgnushack-gnus-load-file
+  (if (featurep 'xemacs)
+      (expand-file-name "auto-autoloads.el" srcdir)
+    (expand-file-name "gnus-load.el" srcdir)))
+
+(defvar	dgnushack-cus-load-file
+  (if (featurep 'xemacs)
+      (expand-file-name "custom-load.el" srcdir)
+    (expand-file-name "cus-load.el" srcdir)))
 
 (defun dgnushack-make-cus-load ()
   (when (condition-case nil
 	    (load "cus-dep")
-	  (error nil))
+	  (error
+	   (when (boundp 'MULE)
+	     (if (file-exists-p "../contrib/cus-dep.el")
+		 ;; Use cus-dep.el of the version of Emacs 20.7.
+		 (load-file "../contrib/cus-dep.el")
+	       (error "\
+You need contrib/cus-dep.el to build T-gnus with Mule 2.3@19.34; exiting.")))))
     (let ((cusload-base-file dgnushack-cus-load-file))
       (if (fboundp 'custom-make-dependencies)
 	  (custom-make-dependencies)
-	(Custom-make-dependencies)))))
+	(Custom-make-dependencies))
+      (when (featurep 'xemacs)
+	(message "Compiling %s..." dgnushack-cus-load-file)
+	(byte-compile-file dgnushack-cus-load-file)))))
 
 (defun dgnushack-make-auto-load ()
   (require 'autoload)
@@ -621,11 +766,8 @@ Modify to suit your needs."))
 	(make-backup-files nil)
 	(autoload-package-name "gnus"))
     (if (featurep 'xemacs)
-	(progn
-	  (if (file-exists-p generated-autoload-file)
-	      (delete-file generated-autoload-file))
-	  (if (file-exists-p dgnushack-auto-load-file)
-	      (delete-file dgnushack-auto-load-file)))
+	(if (file-exists-p generated-autoload-file)
+	    (delete-file generated-autoload-file))
       (with-temp-file generated-autoload-file
 	(insert ?\014)))
     (if (featurep 'xemacs)
@@ -644,55 +786,44 @@ Modify to suit your needs."))
       (batch-update-autoloads))))
 
 (defun dgnushack-make-load ()
-  (message (format "Generating %s..." dgnushack-gnus-load-file))
-  (with-temp-file dgnushack-gnus-load-file
-    (if (file-exists-p dgnushack-cus-load-file)
-	(progn
-	  (insert-file-contents dgnushack-cus-load-file)
-	  (delete-file dgnushack-cus-load-file)
-	  (goto-char (point-min))
-	  (search-forward ";;; Code:")
-	  (forward-line)
-	  (delete-region (point-min) (point))
-	  (unless (re-search-forward "\
-^[\t ]*(autoload[\t\n ]+\\('\\|(quote[\t\n ]+\\)custom-add-loads[\t\n ]"
-				     nil t)
-	    (insert "\n(autoload 'custom-add-loads \"cus-load\")\n"))
-	  (goto-char (point-min))
-	  (insert "\
+  (unless (featurep 'xemacs)
+    (message "Generating %s..." dgnushack-gnus-load-file)
+    (with-temp-file dgnushack-gnus-load-file
+      (insert-file-contents dgnushack-cus-load-file)
+      (delete-file dgnushack-cus-load-file)
+      (goto-char (point-min))
+      (search-forward ";;; Code:")
+      (forward-line)
+      (delete-region (point-min) (point))
+      (insert "\
 ;;; gnus-load.el --- automatically extracted custom dependencies and autoload
 ;;
 ;;; Code:
 ")
-	  (goto-char (point-max))
-	  (if (search-backward "custom-versions-load-alist" nil t)
-	      (forward-line -1)
-	    (forward-line -1)
-	    (while (eq (char-after) ?\;)
-	      (forward-line -1))
-	    (forward-line))
-	  (delete-region (point) (point-max))
-	  (insert "\n"))
+      (goto-char (point-max))
+      (if (search-backward "custom-versions-load-alist" nil t)
+	  (forward-line -1)
+	(forward-line -1)
+	(while (eq (char-after) ?\;)
+	  (forward-line -1))
+	(forward-line))
+      (delete-region (point) (point-max))
+      (insert "\n")
+      ;; smiley-* are duplicated. Remove them all.
+      (let ((point (point)))
+	(insert-file-contents dgnushack-gnus-load-file)
+	(goto-char point)
+	(while (search-forward "smiley-" nil t)
+	  (beginning-of-line)
+	  (if (looking-at "(autoload ")
+	      (delete-region (point) (progn (forward-sexp) (point)))
+	    (forward-line))))
+      ;;
+      (goto-char (point-max))
+      (when (search-backward "\n(provide " nil t)
+	(forward-line -1)
+	(delete-region (point) (point-max)))
       (insert "\
-;;; gnus-load.el --- automatically extracted autoload
-;;
-;;; Code:
-"))
-    ;; smiley-* are duplicated. Remove them all.
-    (let ((point (point)))
-      (insert-file-contents dgnushack-gnus-load-file)
-      (goto-char point)
-      (while (search-forward "smiley-" nil t)
-	(beginning-of-line)
-	(if (looking-at "(autoload ")
-	    (delete-region (point) (progn (forward-sexp) (point)))
-	  (forward-line))))
-    ;;
-    (goto-char (point-max))
-    (when (search-backward "\n(provide " nil t)
-      (forward-line -1)
-      (delete-region (point) (point-max)))
-    (insert "\
 
 \(provide 'gnus-load)
 
@@ -703,72 +834,22 @@ Modify to suit your needs."))
 ;;; End:
 ;;; gnus-load.el ends here
 ")
-    ;; Workaround the bug in some version of XEmacs.
-    (when (featurep 'xemacs)
-      (condition-case nil
-	  (require 'cus-load)
-	(error nil))
-      (goto-char (point-min))
-      (when (and (fboundp 'custom-add-loads)
-		 (not (search-forward "\n(autoload 'custom-add-loads " nil t)))
-	(search-forward "\n;;; Code:" nil t)
-	(forward-line 1)
-	(insert "\n(autoload 'custom-add-loads \"cus-load\")\n"))))
-  (message (format "Compiling %s..." dgnushack-gnus-load-file))
-  (byte-compile-file dgnushack-gnus-load-file))
-
-
-(defun dgnushack-compose-package ()
-  "Re-split the file gnus-load.el into custom-load.el and
-auto-autoloads.el.  It is silly, should be improved!"
-  (message "
-Re-splitting gnus-load.el into custom-load.el and auto-autoloads.el...")
-  (let ((customload (expand-file-name "custom-load.el" srcdir))
-	(autoloads (expand-file-name "auto-autoloads.el" srcdir))
-	start)
-    (with-temp-buffer
-      (insert-file-contents dgnushack-gnus-load-file)
-      (delete-file dgnushack-gnus-load-file)
-      (when (file-exists-p (concat dgnushack-gnus-load-file "c"))
-	(delete-file (concat dgnushack-gnus-load-file "c")))
-      (while (prog1
-		 (looking-at "[\t ;]")
-	       (forward-line 1)))
-      (setq start (point))
+      ))
+  (message "Compiling %s..." dgnushack-gnus-load-file)
+  (byte-compile-file dgnushack-gnus-load-file)
+  (when (featurep 'xemacs)
+    (message "Creating dummy gnus-load.el...")
+    (with-temp-file (expand-file-name "gnus-load.el")
       (insert "\
-;;; custom-load.el --- automatically extracted custom dependencies\n
-;;; Code:\n\n")
-      (goto-char (point-max))
-      (while (progn
-	       (forward-line -1)
-	       (not (looking-at "[\t ]*(custom-add-loads[\t\n ]"))))
-      (forward-list 1)
-      (forward-line 1)
-      (insert "\n;;; custom-load.el ends here\n")
-      (write-region start (point) customload)
-      (while (looking-at "[\t ]*$")
-	(forward-line 1))
-      (setq start (point))
-      (if (re-search-forward "^[\t\n ]*(if[\t\n ]+(featurep[\t\n ]" nil t)
-	  (let ((from (goto-char (match-beginning 0))))
-	    (delete-region from (progn
-				  (forward-list 1)
-				  (forward-line 1)
-				  (point))))
-	(while (looking-at "[\t ;]")
-	  (forward-line 1)))
-      (insert "(if (featurep 'gnus-autoloads) (error \"Already loaded\"))\n")
-      (goto-char (point-max))
-      (while (progn
-	       (forward-line -1)
-	       (not (looking-at "[\t ]*(provide[\t\n ]"))))
-      (insert "(provide 'gnus-autoloads)\n")
-      (write-region start (point) autoloads))
-    (byte-compile-file customload)
-    (byte-compile-file autoloads))
-  (message "\
-Re-splitting gnus-load.el into custom-load.el and auto-autoloads.el...done
-\n"))
+
+\(provide 'gnus-load)
+
+;;; Local Variables:
+;;; version-control: never
+;;; no-byte-compile: t
+;;; no-update-autoloads: t
+;;; End:
+;;; gnus-load.el ends here"))))
 
 
 (defconst dgnushack-info-file-regexp-en
@@ -797,6 +878,7 @@ Re-splitting gnus-load.el into custom-load.el and auto-autoloads.el...done
 					    "/")
 				    ;; PACKAGEDIR
 				    (car command-line-args-left))))
+    (setq command-line-args-left nil)
     (when (file-directory-p lisp-dir)
       (let (files)
 	(dolist (file dgnushack-exporting-files)
@@ -811,11 +893,15 @@ Re-splitting gnus-load.el into custom-load.el and auto-autoloads.el...done
 
 (defun dgnushack-install-package-manifest ()
   "Install MANIFEST file as an XEmacs package."
-  (let* ((package-dir (car command-line-args-left))
-	 (product-name (cadr command-line-args-left))
-	 (name (expand-file-name (concat "pkginfo/MANIFEST." product-name)
-				 package-dir))
+  (let* ((package-dir (pop command-line-args-left))
+	 (product-name (pop command-line-args-left))
+	 (pkginfo-dir (expand-file-name "pkginfo" package-dir))
+	 (name (expand-file-name (concat "MANIFEST." product-name)
+				 pkginfo-dir))
 	 make-backup-files)
+    (unless (file-directory-p pkginfo-dir)
+      (message "Creating directory %s/..." pkginfo-dir)
+      (make-directory pkginfo-dir))
     (message "Generating %s..." name)
     (with-temp-file name
       (insert "pkginfo/MANIFEST." product-name "\n")
