@@ -2,8 +2,6 @@
 ;;
 ;;; Code:
 
-(autoload 'custom-add-loads "cus-load")
-
 (put 'mime-display 'custom-loads '("mm-decode"))
 (put 'spam-stat 'custom-loads '("spam-stat"))
 (put 'bbdb-mua-specific-gnus 'custom-loads '("gnus-bbdb"))
@@ -23,6 +21,7 @@
 (put 'gnus-start 'custom-loads '("gnus-group" "gnus-int" "gnus-start" "gnus-util" "gnus" "nnheader"))
 (put 'gnus-extract-view 'custom-loads '("gnus-sum" "gnus-uu"))
 (put 'gnus-undo 'custom-loads '("gnus-undo"))
+(put 'spam-spamoracle 'custom-loads '("spam"))
 (put 'gnus-various 'custom-loads '("gnus-sum" "gnus-util" "gnus"))
 (put 'gnus-meta 'custom-loads '("gnus"))
 (put 'mail-source 'custom-loads '("mail-source" "nndiary"))
@@ -34,21 +33,22 @@
 (put 'gnus-xmas 'custom-loads '("gnus-xmas"))
 (put 'namazu 'custom-loads '("gnus-namazu"))
 (put 'gnus-summary-pick 'custom-loads '("gnus-salt"))
-(put 'gnus-thread 'custom-loads '("gnus-sum"))
+(put 'gnus-thread 'custom-loads '("gnus-group" "gnus-sum"))
 (put 'languages 'custom-loads '("sieve"))
 (put 'spam 'custom-loads '("spam"))
 (put 'nnmail-various 'custom-loads '("nnmail"))
+(put 'extensions 'custom-loads '("sha1-el"))
 (put 'message-various 'custom-loads '("message"))
 (put 'gnus-summary-exit 'custom-loads '("gnus-sum" "gnus"))
 (put 'news 'custom-loads '("gnus-offline" "gnus" "message" "mm-decode"))
-(put 'gnus 'custom-loads '("gnus-art" "gnus-async" "gnus-delay" "gnus-demon" "gnus-dup" "gnus-eform" "gnus-namazu" "gnus-offline" "gnus-sieve" "gnus-undo" "gnus-uu" "gnus-win" "gnus-xmas" "gnus" "mail-source" "mm-url" "nnimap" "nnmail" "nnshimbun" "spam-stat"))
+(put 'gnus 'custom-loads '("gnus-art" "gnus-async" "gnus-delay" "gnus-demon" "gnus-dup" "gnus-eform" "gnus-namazu" "gnus-offline" "gnus" "gnus-sieve" "gnus-undo" "gnus-uu" "gnus-win" "gnus-xmas" "mail-source" "mm-url" "nnimap" "nnmail" "nnshimbun" "spam-stat"))
 (put 'spam-bogofilter 'custom-loads '("spam"))
 (put 'gnus-server 'custom-loads '("gnus-srvr" "gnus" "nnheader"))
 (put 'multimedia 'custom-loads '("gnus-audio" "mm-decode"))
 (put 'gnus-summary-visual 'custom-loads '("gnus-sum" "gnus"))
 (put 'gnus-group-listing 'custom-loads '("gnus-group"))
 (put 'gnus-score 'custom-loads '("gnus-nocem" "gnus"))
-(put 'gnus-group-select 'custom-loads '("gnus-sum" "gnus"))
+(put 'gnus-group-select 'custom-loads '("gnus-group" "gnus-sum" "gnus"))
 (put 'message-buffers 'custom-loads '("message"))
 (put 'gnus-threading 'custom-loads '("gnus-sum"))
 (put 'gnus-score-decay 'custom-loads '("gnus-score"))
@@ -58,6 +58,7 @@
 (put 'gnus-cite 'custom-loads '("gnus-cite"))
 (put 'gnus-demon 'custom-loads '("gnus-demon"))
 (put 'gnus-message 'custom-loads '("gnus-msg" "message" "gnus"))
+(put 'tls 'custom-loads '("tls"))
 (put 'starttls 'custom-loads '("starttls"))
 (put 'gnus-summary-tree 'custom-loads '("gnus-salt"))
 (put 'gnus-score-default 'custom-loads '("gnus-score" "gnus-sum"))
@@ -74,12 +75,14 @@
 (put 'tools 'custom-loads '("sieve"))
 (put 'gnus-topic 'custom-loads '("gnus-topic"))
 (put 'gnus-summary-choose 'custom-loads '("gnus-sum"))
+(put 'sha1 'custom-loads '("sha1-el"))
 (put 'message-headers 'custom-loads '("message"))
 (put 'bbdb-mua-specific-gnus-scoring 'custom-loads '("gnus-bbdb"))
 (put 'message-forwarding 'custom-loads '("message"))
 (put 'message-faces 'custom-loads '("message"))
 (put 'gnus-article-treat 'custom-loads '("gnus-art"))
 (put 'gnus-charset 'custom-loads '("gnus-group" "gnus-sum" "gnus"))
+(put 'comm 'custom-loads '("tls"))
 (put 'gnus-article-mime 'custom-loads '("gnus-art" "gnus-sum" "mm-uu"))
 (put 'nnmail-retrieve 'custom-loads '("nnmail"))
 (put 'gnus-duplicate 'custom-loads '("gnus-dup"))
@@ -94,7 +97,7 @@
 (put 'gnus-windows 'custom-loads '("gnus-win"))
 (put 'gnus-article-buttons 'custom-loads '("gnus-art"))
 (put 'gnus-summary 'custom-loads '("gnus-sum" "gnus"))
-(put 'gnus-article-hiding 'custom-loads '("gnus-art" "gnus-sum"))
+(put 'gnus-article-hiding 'custom-loads '("gnus-art" "gnus-cite" "gnus-sum"))
 (put 'gnus-group 'custom-loads '("gnus-topic" "gnus"))
 (put 'gnus-article-various 'custom-loads '("gnus-art" "gnus-sum"))
 (put 'imap 'custom-loads '("imap"))
@@ -105,10 +108,11 @@
 (put 'faces 'custom-loads '("gnus" "message"))
 (put 'gnus-summary-various 'custom-loads '("gnus-sum"))
 (put 'applications 'custom-loads '("canlock"))
+(put 'gnus-registry 'custom-loads '("gnus-registry"))
 (put 'gnus-start-server 'custom-loads '("gnus-start"))
 (put 'canlock 'custom-loads '("canlock-om" "canlock"))
 (put 'gnus-extract-archive 'custom-loads '("gnus-uu"))
-(put 'message 'custom-loads '("message" "mml"))
+(put 'message 'custom-loads '("message" "mml-sec" "mml"))
 (put 'message-sending 'custom-loads '("message"))
 (put 'gnus-group-icons 'custom-loads '("gnus-group"))
 (put 'gnus-score-adapt 'custom-loads '("gnus-score" "gnus"))
@@ -127,8 +131,9 @@
 (put 'gnus-picon 'custom-loads '("gnus-art" "gnus-picon"))
 (put 'gnus-extract 'custom-loads '("binhex" "gnus" "gnus-uu" "uudecode"))
 (put 'gnus-outlook-deuglify 'custom-loads '("deuglify"))
-(put 'gnus-article 'custom-loads '("gnus-art" "gnus-cite" "gnus-sum"))
+(put 'gnus-article 'custom-loads '("gnus-art" "gnus-cite"))
 (put 'nnimap 'custom-loads '("nnimap"))
+(put 'spam-report 'custom-loads '("spam-report"))
 (put 'nnir 'custom-loads '("nnir"))
 (put 'gnus-group-foreign 'custom-loads '("gnus-group" "gnus-msg"))
 (put 'gnus-offline 'custom-loads '("gnus-offline"))
@@ -184,6 +189,8 @@
 (custom-put-if-not 'gnus-article-loose-mime 'standard-value t)
 (custom-put-if-not 'gnus-debug-files 'custom-version "21.1")
 (custom-put-if-not 'gnus-debug-files 'standard-value t)
+(custom-put-if-not 'mm-enable-external 'custom-version "21.4")
+(custom-put-if-not 'mm-enable-external 'standard-value t)
 (custom-put-if-not 'message-wide-reply-confirm-recipients 'custom-version "21.3")
 (custom-put-if-not 'message-wide-reply-confirm-recipients 'standard-value t)
 (custom-put-if-not 'gnus-article-banner-alist 'custom-version "21.1")
@@ -230,8 +237,6 @@
 (custom-put-if-not 'nnmail-extra-headers 'standard-value t)
 (custom-put-if-not 'gnus-mime-multipart-functions 'custom-version "21.1")
 (custom-put-if-not 'gnus-mime-multipart-functions 'standard-value t)
-(custom-put-if-not 'gnus-treat-display-grey-xface 'custom-version "21.3")
-(custom-put-if-not 'gnus-treat-display-grey-xface 'standard-value t)
 (custom-put-if-not 'gnus-charset 'custom-version "21.1")
 (custom-put-if-not 'gnus-charset 'group-documentation "Group character set issues.")
 (custom-put-if-not 'gnus-nocem-check-from 'custom-version "21.1")
@@ -285,8 +290,8 @@
 
 
 ;;;### (autoloads (binhex-decode-region binhex-decode-region-external
-;;;;;;  binhex-decode-region-internal) "binhex" "binhex.el" (15916
-;;;;;;  61430))
+;;;;;;  binhex-decode-region-internal) "binhex" "binhex.el" (16049
+;;;;;;  55520))
 ;;; Generated autoloads from binhex.el
 
 (defconst binhex-begin-line "^:...............................................................$")
@@ -304,7 +309,7 @@ Binhex decode region between START and END." t nil)
 ;;;***
 
 ;;;### (autoloads (canlock-verify canlock-insert-header) "canlock"
-;;;;;;  "canlock.el" (15575 16891))
+;;;;;;  "canlock.el" (16376 42740))
 ;;; Generated autoloads from canlock.el
 
 (autoload (quote canlock-insert-header) "canlock" "\
@@ -323,7 +328,7 @@ nil instead of to signal an error by setting the option
 ;;;;;;  gnus-article-outlook-repair-attribution gnus-article-outlook-unwrap-lines
 ;;;;;;  gnus-outlook-display-hook gnus-outlook-deuglify-unwrap-max
 ;;;;;;  gnus-outlook-deuglify-unwrap-min) "deuglify" "deuglify.el"
-;;;;;;  (15916 61431))
+;;;;;;  (16376 42740))
 ;;; Generated autoloads from deuglify.el
 
 (defvar gnus-outlook-deuglify-unwrap-min 45 "\
@@ -337,10 +342,10 @@ A hook called after an deuglified article has been prepared.
 It is run after `gnus-article-prepare-hook'.")
 
 (autoload (quote gnus-article-outlook-unwrap-lines) "deuglify" "\
-Unwrap lines that appear to be wrapped citation lines.  
+Unwrap lines that appear to be wrapped citation lines.
 You can control what lines will be unwrapped by frobbing
 `gnus-outlook-deuglify-unwrap-min' and `gnus-outlook-deuglify-unwrap-max',
-indicating the miminum and maximum length of an unwrapped citation line.  If
+indicating the minimum and maximum length of an unwrapped citation line.  If
 NODISPLAY is non-nil, don't redisplay the article buffer." t nil)
 
 (autoload (quote gnus-article-outlook-repair-attribution) "deuglify" "\
@@ -357,17 +362,27 @@ Deuglify broken Outlook (Express) articles and redisplay." t nil)
 
 ;;;***
 
-;;;### (autoloads (gnus-earcon-display) "earcon" "earcon.el" (15418
-;;;;;;  33141))
+;;;### (autoloads (gnus-earcon-display) "earcon" "earcon.el" (16049
+;;;;;;  55521))
 ;;; Generated autoloads from earcon.el
 
 (autoload (quote gnus-earcon-display) "earcon" "\
 Play sounds in message buffers." t nil)
 
+;;;***
+
+;;;### (autoloads (fill-flowed fill-flowed-encode) "flow-fill" "flow-fill.el"
+;;;;;;  (16376 42741))
+;;; Generated autoloads from flow-fill.el
+
+(autoload (quote fill-flowed-encode) "flow-fill" nil nil nil)
+
+(autoload (quote fill-flowed) "flow-fill" nil nil nil)
+
 ;;;***
 
 ;;;### (autoloads (gnus gnus-other-frame gnus-slave gnus-no-server
-;;;;;;  gnus-slave-no-server) "gnus" "gnus.el" (15916 61435))
+;;;;;;  gnus-slave-no-server) "gnus" "gnus.el" (16376 42762))
 ;;; Generated autoloads from gnus.el
 
 (autoload (quote gnus-slave-no-server) "gnus" "\
@@ -398,7 +413,7 @@ current display is used." t nil)
 (autoload (quote gnus) "gnus" "\
 Read network news.
 If ARG is non-nil and a positive number, Gnus will use that as the
-startup level.	If ARG is non-nil and not a positive number, Gnus will
+startup level.  If ARG is non-nil and not a positive number, Gnus will
 prompt the user for the name of an NNTP server to use." t nil)
 
 ;;;***
@@ -406,7 +421,7 @@ prompt the user for the name of an NNTP server to use." t nil)
 ;;;### (autoloads (gnus-agent-regenerate gnus-agent-batch gnus-agent-batch-fetch
 ;;;;;;  gnus-agent-possibly-save-gcc gnus-agentize gnus-slave-unplugged
 ;;;;;;  gnus-plugged gnus-unplugged) "gnus-agent" "gnus-agent.el"
-;;;;;;  (15916 61431))
+;;;;;;  (16376 42741))
 ;;; Generated autoloads from gnus-agent.el
 
 (autoload (quote gnus-unplugged) "gnus-agent" "\
@@ -420,10 +435,10 @@ Read news as a slave unplugged." t nil)
 
 (autoload (quote gnus-agentize) "gnus-agent" "\
 Allow Gnus to be an offline newsreader.
-The normal usage of this command is to put the following as the
-last form in your `.gnus.el' file:
 
-\(gnus-agentize)
+The gnus-agentize function is now called internally by gnus when
+gnus-agent is set.  If you wish to avoid calling gnus-agentize,
+customize gnus-agent to nil.
 
 This will modify the `gnus-setup-news-hook', and
 `message-send-mail-real-function' variables, and install the Gnus agent
@@ -440,12 +455,12 @@ Start Gnus, send queue and fetch session." t nil)
 
 (autoload (quote gnus-agent-regenerate) "gnus-agent" "\
 Regenerate all agent covered files.
-If CLEAN, don't read existing active files." t nil)
+If CLEAN, obsolete (ignore)." t nil)
 
 ;;;***
 
 ;;;### (autoloads (gnus-article-prepare-display) "gnus-art" "gnus-art.el"
-;;;;;;  (15916 61431))
+;;;;;;  (16376 42744))
 ;;; Generated autoloads from gnus-art.el
 
 (autoload (quote gnus-article-prepare-display) "gnus-art" "\
@@ -454,7 +469,7 @@ Make the current buffer look like a nice article." nil nil)
 ;;;***
 
 ;;;### (autoloads (gnus-audio-play) "gnus-audio" "gnus-audio.el"
-;;;;;;  (15916 61432))
+;;;;;;  (16049 55528))
 ;;; Generated autoloads from gnus-audio.el
 
 (autoload (quote gnus-audio-play) "gnus-audio" "\
@@ -465,7 +480,7 @@ Play a sound FILE through the speaker." t nil)
 ;;;### (autoloads (gnus-bbdb-insinuate-message gnus-bbdb-insinuate
 ;;;;;;  gnus-bbdb/score gnus-bbdb/lines-and-from gnus-bbdb/split-mail
 ;;;;;;  gnus-bbdb/show-sender gnus-bbdb/annotate-sender gnus-bbdb/update-record)
-;;;;;;  "gnus-bbdb" "gnus-bbdb.el" (15575 16892))
+;;;;;;  "gnus-bbdb" "gnus-bbdb.el" (16049 55528))
 ;;; Generated autoloads from gnus-bbdb.el
 
 (autoload (quote gnus-bbdb/update-record) "gnus-bbdb" "\
@@ -520,7 +535,7 @@ Call this function to hook BBDB into message-mode." nil nil)
 ;;;***
 
 ;;;### (autoloads (gnus-cache-generate-nov-databases gnus-cache-generate-active
-;;;;;;  gnus-jog-cache) "gnus-cache" "gnus-cache.el" (15895 52375))
+;;;;;;  gnus-jog-cache) "gnus-cache" "gnus-cache.el" (16376 42749))
 ;;; Generated autoloads from gnus-cache.el
 
 (autoload (quote gnus-jog-cache) "gnus-cache" "\
@@ -538,7 +553,7 @@ Generate NOV files recursively starting in DIR." t nil)
 ;;;***
 
 ;;;### (autoloads (find-cl-run-time-functions) "gnus-clfns" "gnus-clfns.el"
-;;;;;;  (15439 59183))
+;;;;;;  (16049 55529))
 ;;; Generated autoloads from gnus-clfns.el
 
 (autoload (quote find-cl-run-time-functions) "gnus-clfns" "\
@@ -556,7 +571,7 @@ You can use the `digit-argument' 1, 2 or 3 instead of \\[universal-argument]'s."
 ;;;***
 
 ;;;### (autoloads (gnus-delay-initialize gnus-delay-send-queue gnus-delay-article)
-;;;;;;  "gnus-delay" "gnus-delay.el" (15695 54011))
+;;;;;;  "gnus-delay" "gnus-delay.el" (16049 56069))
 ;;; Generated autoloads from gnus-delay.el
 
 (defgroup gnus-delay nil "Arrange for sending postings later." :group (quote gnus))
@@ -588,7 +603,7 @@ Checking delayed messages is skipped if optional arg NO-CHECK is non-nil." nil n
 ;;;***
 
 ;;;### (autoloads (turn-on-gnus-dired-mode) "gnus-dired" "gnus-dired.el"
-;;;;;;  (15575 16892))
+;;;;;;  (16376 42749))
 ;;; Generated autoloads from gnus-dired.el
 
 (autoload (quote turn-on-gnus-dired-mode) "gnus-dired" "\
@@ -597,7 +612,7 @@ Convenience method to turn on gnus-dired-mode." nil nil)
 ;;;***
 
 ;;;### (autoloads (gnus-draft-reminder) "gnus-draft" "gnus-draft.el"
-;;;;;;  (15895 52375))
+;;;;;;  (16376 42749))
 ;;; Generated autoloads from gnus-draft.el
 
 (autoload (quote gnus-draft-reminder) "gnus-draft" "\
@@ -605,10 +620,9 @@ Reminder user if there are unsent drafts." t nil)
 
 ;;;***
 
-;;;### (autoloads (gnus-convert-gray-x-face-region gnus-convert-gray-x-face-to-xpm
-;;;;;;  gnus-convert-face-to-png gnus-face-from-file gnus-x-face-from-file
-;;;;;;  gnus-insert-random-x-face-header gnus-random-x-face) "gnus-fun"
-;;;;;;  "gnus-fun.el" (15916 61432))
+;;;### (autoloads (gnus-convert-png-to-face gnus-convert-face-to-png
+;;;;;;  gnus-face-from-file gnus-x-face-from-file gnus-insert-random-x-face-header
+;;;;;;  gnus-random-x-face) "gnus-fun" "gnus-fun.el" (16376 42750))
 ;;; Generated autoloads from gnus-fun.el
 
 (autoload (quote gnus-random-x-face) "gnus-fun" "\
@@ -623,17 +637,19 @@ Insert an X-Face header based on an image file." t nil)
 (autoload (quote gnus-face-from-file) "gnus-fun" "\
 Return an Face header based on an image file." t nil)
 
-(autoload (quote gnus-convert-face-to-png) "gnus-fun" nil nil nil)
+(autoload (quote gnus-convert-face-to-png) "gnus-fun" "\
+Convert FACE (which is base64-encoded) to a PNG.
+The PNG is returned as a string." nil nil)
 
-(autoload (quote gnus-convert-gray-x-face-to-xpm) "gnus-fun" nil nil nil)
-
-(autoload (quote gnus-convert-gray-x-face-region) "gnus-fun" "\
-Convert the X-Faces in region to a PPM file." t nil)
+(autoload (quote gnus-convert-png-to-face) "gnus-fun" "\
+Convert FILE to a Face.
+FILE should be a PNG file that's 48x48 and smaller than or equal to
+726 bytes." nil nil)
 
 ;;;***
 
 ;;;### (autoloads (gnus-fetch-group-other-frame gnus-fetch-group)
-;;;;;;  "gnus-group" "gnus-group.el" (15916 61433))
+;;;;;;  "gnus-group" "gnus-group.el" (16376 42750))
 ;;; Generated autoloads from gnus-group.el
 
 (autoload (quote gnus-fetch-group) "gnus-group" "\
@@ -645,8 +661,8 @@ Pop up a frame and enter GROUP." t nil)
 
 ;;;***
 
-;;;### (autoloads (gnus-batch-score) "gnus-kill" "gnus-kill.el" (15575
-;;;;;;  16892))
+;;;### (autoloads (gnus-batch-score) "gnus-kill" "gnus-kill.el" (16049
+;;;;;;  55532))
 ;;; Generated autoloads from gnus-kill.el
 
 (defalias (quote gnus-batch-kill) (quote gnus-batch-score))
@@ -658,8 +674,8 @@ Usage: emacs -batch -l ~/.emacs -l gnus -f gnus-batch-score" t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-mailing-list-mode gnus-mailing-list-insinuate
-;;;;;;  turn-on-gnus-mailing-list-mode) "gnus-ml" "gnus-ml.el" (15418
-;;;;;;  33142))
+;;;;;;  turn-on-gnus-mailing-list-mode) "gnus-ml" "gnus-ml.el" (16049
+;;;;;;  55534))
 ;;; Generated autoloads from gnus-ml.el
 
 (autoload (quote turn-on-gnus-mailing-list-mode) "gnus-ml" nil nil nil)
@@ -676,8 +692,8 @@ Minor mode for providing mailing-list commands.
 ;;;***
 
 ;;;### (autoloads (gnus-group-split-fancy gnus-group-split gnus-group-split-update
-;;;;;;  gnus-group-split-setup) "gnus-mlspl" "gnus-mlspl.el" (15895
-;;;;;;  52376))
+;;;;;;  gnus-group-split-setup) "gnus-mlspl" "gnus-mlspl.el" (16049
+;;;;;;  55534))
 ;;; Generated autoloads from gnus-mlspl.el
 
 (autoload (quote gnus-group-split-setup) "gnus-mlspl" "\
@@ -769,7 +785,7 @@ Calling (gnus-group-split-fancy nil nil \"mail.others\") returns:
 ;;;***
 
 ;;;### (autoloads (gnus-change-server) "gnus-move" "gnus-move.el"
-;;;;;;  (15066 10800))
+;;;;;;  (15066 10799))
 ;;; Generated autoloads from gnus-move.el
 
 (autoload (quote gnus-change-server) "gnus-move" "\
@@ -779,7 +795,7 @@ Update the .newsrc.eld file to reflect the change of nntp server." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-button-reply gnus-button-mailto gnus-msg-mail)
-;;;;;;  "gnus-msg" "gnus-msg.el" (15916 61433))
+;;;;;;  "gnus-msg" "gnus-msg.el" (16376 42751))
 ;;; Generated autoloads from gnus-msg.el
 
 (autoload (quote gnus-msg-mail) "gnus-msg" "\
@@ -799,8 +815,8 @@ Like `message-reply'." t nil)
 
 ;;;### (autoloads (gnus-namazu-insinuate gnus-namazu-stop-update
 ;;;;;;  gnus-namazu-update-all-indices gnus-namazu-update-index gnus-namazu-create-index
-;;;;;;  gnus-namazu-search) "gnus-namazu" "gnus-namazu.el" (15895
-;;;;;;  52376))
+;;;;;;  gnus-namazu-search) "gnus-namazu" "gnus-namazu.el" (16376
+;;;;;;  42752))
 ;;; Generated autoloads from gnus-namazu.el
 
 (autoload (quote gnus-namazu-search) "gnus-namazu" "\
@@ -824,8 +840,8 @@ Stop the running indexer of Namazu." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-treat-newsgroups-picon gnus-treat-mail-picon
-;;;;;;  gnus-treat-from-picon) "gnus-picon" "gnus-picon.el" (15435
-;;;;;;  43748))
+;;;;;;  gnus-treat-from-picon) "gnus-picon" "gnus-picon.el" (16376
+;;;;;;  42753))
 ;;; Generated autoloads from gnus-picon.el
 
 (autoload (quote gnus-treat-from-picon) "gnus-picon" "\
@@ -845,7 +861,7 @@ If picons are already displayed, remove them." t nil)
 ;;;### (autoloads (gnus-add-to-sorted-list gnus-sorted-nunion gnus-sorted-union
 ;;;;;;  gnus-sorted-nintersection gnus-sorted-intersection gnus-intersection
 ;;;;;;  gnus-sorted-complement gnus-sorted-ndifference gnus-sorted-difference)
-;;;;;;  "gnus-range" "gnus-range.el" (15895 52376))
+;;;;;;  "gnus-range" "gnus-range.el" (16049 55537))
 ;;; Generated autoloads from gnus-range.el
 
 (autoload (quote gnus-sorted-difference) "gnus-range" "\
@@ -885,10 +901,21 @@ LIST1 and LIST2 have to be sorted over <." nil nil)
 (autoload (quote gnus-add-to-sorted-list) "gnus-range" "\
 Add NUM into sorted LIST by side effect." nil nil)
 
+;;;***
+
+;;;### (autoloads (gnus-registry-install-hooks gnus-registry-initialize)
+;;;;;;  "gnus-registry" "gnus-registry.el" (16376 42753))
+;;; Generated autoloads from gnus-registry.el
+
+(autoload (quote gnus-registry-initialize) "gnus-registry" nil t nil)
+
+(autoload (quote gnus-registry-install-hooks) "gnus-registry" "\
+Install the registry hooks." t nil)
+
 ;;;***
 
 ;;;### (autoloads (gnus-sieve-article-add-rule gnus-sieve-generate
-;;;;;;  gnus-sieve-update) "gnus-sieve" "gnus-sieve.el" (15905 32041))
+;;;;;;  gnus-sieve-update) "gnus-sieve" "gnus-sieve.el" (16376 42754))
 ;;; Generated autoloads from gnus-sieve.el
 
 (autoload (quote gnus-sieve-update) "gnus-sieve" "\
@@ -909,7 +936,7 @@ See the documentation for these variables and functions for details." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-batch-brew-soup) "gnus-soup" "gnus-soup.el"
-;;;;;;  (15895 52376))
+;;;;;;  (16376 42754))
 ;;; Generated autoloads from gnus-soup.el
 
 (autoload (quote gnus-batch-brew-soup) "gnus-soup" "\
@@ -927,7 +954,7 @@ Note -- this function hasn't been implemented yet." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-update-format) "gnus-spec" "gnus-spec.el"
-;;;;;;  (15905 32041))
+;;;;;;  (16049 55539))
 ;;; Generated autoloads from gnus-spec.el
 
 (autoload (quote gnus-update-format) "gnus-spec" "\
@@ -936,25 +963,19 @@ Update the format specification near point." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-fixup-nnimap-unread-after-getting-new-news
-;;;;;;  gnus-declare-backend gnus-unload) "gnus-start" "gnus-start.el"
-;;;;;;  (15905 8931))
+;;;;;;  gnus-declare-backend) "gnus-start" "gnus-start.el" (16376
+;;;;;;  42754))
 ;;; Generated autoloads from gnus-start.el
 
-(autoload (quote gnus-unload) "gnus-start" "\
-Unload all Gnus features.
-\(For some value of `all' or `Gnus'.)  Currently, features whose names
-have prefixes `gnus-', `nn', `mm-' or `rfc' are unloaded.  Use
-cautiously -- unloading may cause trouble." t nil)
-
 (autoload (quote gnus-declare-backend) "gnus-start" "\
-Declare backend NAME with ABILITIES as a Gnus backend." nil nil)
+Declare back end NAME with ABILITIES as a Gnus back end." nil nil)
 
 (autoload (quote gnus-fixup-nnimap-unread-after-getting-new-news) "gnus-start" nil nil nil)
 
 ;;;***
 
 ;;;### (autoloads (gnus-add-configuration) "gnus-win" "gnus-win.el"
-;;;;;;  (15916 61435))
+;;;;;;  (16049 55543))
 ;;; Generated autoloads from gnus-win.el
 
 (autoload (quote gnus-add-configuration) "gnus-win" "\
@@ -962,7 +983,7 @@ Add the window configuration CONF to `gnus-buffer-configuration'." nil nil)
 
 ;;;***
 
-;;;### (autoloads (html2text) "html2text" "html2text.el" (15575 16896))
+;;;### (autoloads (html2text) "html2text" "html2text.el" (16376 42763))
 ;;; Generated autoloads from html2text.el
 
 (autoload (quote html2text) "html2text" "\
@@ -986,8 +1007,8 @@ Convert HTML to plain text in the current buffer." t nil)
 ;;;;;;  message-signature-separator message-from-style message-cross-post-note-function
 ;;;;;;  message-followup-to-note message-cross-post-note message-cross-post-default
 ;;;;;;  message-archive-note message-archive-header message-mark-insert-end
-;;;;;;  message-mark-insert-begin) "message" "message.el" (15916
-;;;;;;  61436))
+;;;;;;  message-mark-insert-begin) "message" "message.el" (16376
+;;;;;;  42764))
 ;;; Generated autoloads from message.el
 
 (defvar message-mark-insert-begin "--8<---------------cut here---------------start------------->8---\n" "\
@@ -998,7 +1019,7 @@ How to mark the end of some inserted text.")
 
 (defvar message-archive-header "X-No-Archive: Yes\n" "\
 Header to insert when you don't want your article to be archived.
-Archives (such as groups.googgle.com) respect this header.")
+Archives (such as groups.google.com) respect this header.")
 
 (defvar message-archive-note "X-No-Archive: Yes - save http://groups.google.com/" "\
 Note to insert why you wouldn't want this posting archived.
@@ -1214,7 +1235,7 @@ Optional NEWS will use news to forward instead of mail." t nil)
 (autoload (quote message-forward-rmail-make-body) "message" nil nil nil)
 
 (autoload (quote message-insinuate-rmail) "message" "\
-Let RMAIL uses message to forward." t nil)
+Let RMAIL use message to forward." t nil)
 
 (autoload (quote message-resend) "message" "\
 Resend the current article to ADDRESS." t nil)
@@ -1251,7 +1272,7 @@ which specify the range to operate on." t nil)
 ;;;***
 
 ;;;### (autoloads (mm-inline-external-body) "mm-extern" "mm-extern.el"
-;;;;;;  (15418 33147))
+;;;;;;  (16376 42767))
 ;;; Generated autoloads from mm-extern.el
 
 (autoload (quote mm-inline-external-body) "mm-extern" "\
@@ -1263,7 +1284,7 @@ If NO-DISPLAY is nil, display it. Otherwise, do nothing after replacing." nil ni
 ;;;***
 
 ;;;### (autoloads (mm-inline-partial) "mm-partial" "mm-partial.el"
-;;;;;;  (15575 16898))
+;;;;;;  (16049 55551))
 ;;; Generated autoloads from mm-partial.el
 
 (autoload (quote mm-inline-partial) "mm-partial" "\
@@ -1274,7 +1295,20 @@ If NO-DISPLAY is nil, display it. Otherwise, do nothing after replacing." nil ni
 
 ;;;***
 
-;;;### (autoloads (mm-uu-dissect) "mm-uu" "mm-uu.el" (15895 52382))
+;;;### (autoloads (mm-url-insert-file-contents-external mm-url-insert-file-contents)
+;;;;;;  "mm-url" "mm-url.el" (16376 42767))
+;;; Generated autoloads from mm-url.el
+
+(autoload (quote mm-url-insert-file-contents) "mm-url" "\
+Insert file contents of URL.
+If `mm-url-use-external' is non-nil, use `mm-url-program'." nil nil)
+
+(autoload (quote mm-url-insert-file-contents-external) "mm-url" "\
+Insert file contents of URL using `mm-url-program'." nil nil)
+
+;;;***
+
+;;;### (autoloads (mm-uu-dissect) "mm-uu" "mm-uu.el" (16376 42768))
 ;;; Generated autoloads from mm-uu.el
 
 (autoload (quote mm-uu-dissect) "mm-uu" "\
@@ -1283,7 +1317,7 @@ Dissect the current buffer and return a list of uu handles." nil nil)
 ;;;***
 
 ;;;### (autoloads (mml1991-sign mml1991-encrypt) "mml1991" "mml1991.el"
-;;;;;;  (15895 52383))
+;;;;;;  (16376 42769))
 ;;; Generated autoloads from mml1991.el
 
 (autoload (quote mml1991-encrypt) "mml1991" nil nil nil)
@@ -1294,7 +1328,7 @@ Dissect the current buffer and return a list of uu handles." nil nil)
 
 ;;;### (autoloads (mml2015-self-encrypt mml2015-sign mml2015-encrypt
 ;;;;;;  mml2015-verify-test mml2015-verify mml2015-decrypt-test mml2015-decrypt)
-;;;;;;  "mml2015" "mml2015.el" (15895 52383))
+;;;;;;  "mml2015" "mml2015.el" (16376 42769))
 ;;; Generated autoloads from mml2015.el
 
 (autoload (quote mml2015-decrypt) "mml2015" nil nil nil)
@@ -1314,7 +1348,7 @@ Dissect the current buffer and return a list of uu handles." nil nil)
 ;;;***
 
 ;;;### (autoloads (nndiary-generate-nov-databases) "nndiary" "nndiary.el"
-;;;;;;  (15895 52383))
+;;;;;;  (16376 42771))
 ;;; Generated autoloads from nndiary.el
 
 (autoload (quote nndiary-generate-nov-databases) "nndiary" "\
@@ -1322,7 +1356,7 @@ Generate NOV databases in all nndiary directories." t nil)
 
 ;;;***
 
-;;;### (autoloads (nndoc-add-type) "nndoc" "nndoc.el" (15695 54015))
+;;;### (autoloads (nndoc-add-type) "nndoc" "nndoc.el" (16376 42772))
 ;;; Generated autoloads from nndoc.el
 
 (autoload (quote nndoc-add-type) "nndoc" "\
@@ -1335,7 +1369,7 @@ symbol in the alist." nil nil)
 ;;;***
 
 ;;;### (autoloads (nnfolder-generate-active-file) "nnfolder" "nnfolder.el"
-;;;;;;  (15916 61444))
+;;;;;;  (16376 42772))
 ;;; Generated autoloads from nnfolder.el
 
 (autoload (quote nnfolder-generate-active-file) "nnfolder" "\
@@ -1345,7 +1379,7 @@ This command does not work if you use short group names." t nil)
 ;;;***
 
 ;;;### (autoloads (nnkiboze-generate-groups) "nnkiboze" "nnkiboze.el"
-;;;;;;  (15575 16905))
+;;;;;;  (16049 55554))
 ;;; Generated autoloads from nnkiboze.el
 
 (autoload (quote nnkiboze-generate-groups) "nnkiboze" "\
@@ -1355,7 +1389,7 @@ Finds out what articles are to be part of the nnkiboze groups." t nil)
 ;;;***
 
 ;;;### (autoloads (nnml-generate-nov-databases) "nnml" "nnml.el"
-;;;;;;  (15895 52391))
+;;;;;;  (16376 42774))
 ;;; Generated autoloads from nnml.el
 
 (autoload (quote nnml-generate-nov-databases) "nnml" "\
@@ -1364,7 +1398,7 @@ Generate NOV databases in all nnml directories." t nil)
 ;;;***
 
 ;;;### (autoloads (gnus-group-make-shimbun-group) "nnshimbun" "nnshimbun.el"
-;;;;;;  (15895 52391))
+;;;;;;  (16049 55559))
 ;;; Generated autoloads from nnshimbun.el
 
 (autoload (quote gnus-group-make-shimbun-group) "nnshimbun" "\
@@ -1373,7 +1407,7 @@ Create a nnshimbun group." t nil)
 ;;;***
 
 ;;;### (autoloads (nnsoup-revert-variables nnsoup-set-variables nnsoup-pack-replies)
-;;;;;;  "nnsoup" "nnsoup.el" (15916 61445))
+;;;;;;  "nnsoup" "nnsoup.el" (16049 55560))
 ;;; Generated autoloads from nnsoup.el
 
 (autoload (quote nnsoup-pack-replies) "nnsoup" "\
@@ -1385,10 +1419,28 @@ Use the SOUP methods for posting news and mailing mail." t nil)
 (autoload (quote nnsoup-revert-variables) "nnsoup" "\
 Revert posting and mailing methods to the standard Emacs methods." t nil)
 
+;;;***
+
+;;;### (autoloads (quoted-printable-decode-region) "qp" "qp.el" (16376
+;;;;;;  42775))
+;;; Generated autoloads from qp.el
+
+(autoload (quote quoted-printable-decode-region) "qp" "\
+Decode quoted-printable in the region between FROM and TO, per RFC 2045.
+If CODING-SYSTEM is non-nil, decode bytes into characters with that
+coding-system.
+
+Interactively, you can supply the CODING-SYSTEM argument
+with \\[universal-coding-system-argument].
+
+The CODING-SYSTEM argument is a historical hangover and is deprecated.
+QP encodes raw bytes and should be decoded into raw bytes.  Decoding
+them into characters should be done separately." t nil)
+
 ;;;***
 
 ;;;### (autoloads (gnus-score-mode) "score-mode" "score-mode.el"
-;;;;;;  (15418 33156))
+;;;;;;  (16049 55565))
 ;;; Generated autoloads from score-mode.el
 
 (autoload (quote gnus-score-mode) "score-mode" "\
@@ -1397,10 +1449,21 @@ This mode is an extended emacs-lisp mode.
 
 \\{gnus-score-mode-map}" t nil)
 
+;;;***
+
+;;;### (autoloads (sha1) "sha1-el" "sha1-el.el" (16376 42775))
+;;; Generated autoloads from sha1-el.el
+
+(autoload (quote sha1) "sha1-el" "\
+Return the SHA1 (Secure Hash Algorithm) of an object.
+OBJECT is either a string or a buffer.
+Optional arguments BEG and END denote buffer positions for computing the
+hash of a portion of OBJECT." nil nil)
+
 ;;;***
 
 ;;;### (autoloads (sieve-upload-and-bury sieve-upload sieve-manage)
-;;;;;;  "sieve" "sieve.el" (15905 64426))
+;;;;;;  "sieve" "sieve.el" (16376 42775))
 ;;; Generated autoloads from sieve.el
 
 (autoload (quote sieve-manage) "sieve" nil t nil)
@@ -1411,8 +1474,8 @@ This mode is an extended emacs-lisp mode.
 
 ;;;***
 
-;;;### (autoloads (sieve-mode) "sieve-mode" "sieve-mode.el" (15905
-;;;;;;  64425))
+;;;### (autoloads (sieve-mode) "sieve-mode" "sieve-mode.el" (16049
+;;;;;;  56074))
 ;;; Generated autoloads from sieve-mode.el
 
 (autoload (quote sieve-mode) "sieve-mode" "\
@@ -1425,17 +1488,28 @@ Turning on Sieve mode runs `sieve-mode-hook'." t nil)
 
 ;;;***
 
-;;;### (autoloads (smiley-region) "smiley" "smiley.el" (15575 16907))
+;;;### (autoloads (smiley-buffer smiley-region) "smiley" "smiley.el"
+;;;;;;  (16376 42775))
 ;;; Generated autoloads from smiley.el
 
 
 
+
+
+;;;***
+
+;;;### (autoloads (spam-initialize) "spam" "spam.el" (16376 42776))
+;;; Generated autoloads from spam.el
+
+(autoload (quote spam-initialize) "spam" "\
+Install the spam.el hooks and do other initialization" t nil)
+
 ;;;***
 
 ;;;### (autoloads (safe-date-to-time time-to-days time-to-day-in-year
 ;;;;;;  date-leap-year-p days-between date-to-day time-add time-subtract
 ;;;;;;  time-since days-to-time time-less-p seconds-to-time date-to-time)
-;;;;;;  "time-date" "time-date.el" (15895 52392))
+;;;;;;  "time-date" "time-date.el" (16049 55566))
 ;;; Generated autoloads from time-date.el
 
 (autoload (quote date-to-time) "time-date" "\
@@ -1490,7 +1564,7 @@ If DATE is malformed, return a time value of zeros." nil nil)
 
 ;;;### (autoloads (uudecode-decode-region uudecode-decode-region-internal
 ;;;;;;  uudecode-decode-region-external) "uudecode" "uudecode.el"
-;;;;;;  (15916 61445))
+;;;;;;  (16049 55566))
 ;;; Generated autoloads from uudecode.el
 
 (autoload (quote uudecode-decode-region-external) "uudecode" "\
@@ -1509,7 +1583,7 @@ If FILE-NAME is non-nil, save the result to FILE-NAME." nil nil)
 ;;;***
 
 ;;;### (autoloads (yenc-extract-filename yenc-decode-region) "yenc"
-;;;;;;  "yenc.el" (15895 52392))
+;;;;;;  "yenc.el" (16049 56075))
 ;;; Generated autoloads from yenc.el
 
 (autoload (quote yenc-decode-region) "yenc" "\
