@@ -1,5 +1,5 @@
 ;;; gnus-msg.el --- mail and post interface for Semi-gnus
-;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003
+;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004
 ;;        Free Software Foundation, Inc.
 
 ;; Author: Masanobu UMEDA <umerin@flab.flab.fujitsu.junet>
@@ -38,6 +38,7 @@
 (require 'gnus-ems)
 (require 'message)
 (require 'gnus-art)
+(require 'gnus-util)
 
 (defcustom gnus-post-method 'current
   "*Preferred method for posting USENET news.
@@ -74,12 +75,13 @@ of names)."
 		 (function)))
 
 (defcustom gnus-mailing-list-groups nil
-  "*Regexp matching groups that are really mailing lists.
+  "*If non-nil a regexp matching groups that are really mailing lists.
 This is useful when you're reading a mailing list that has been
 gatewayed to a newsgroup, and you want to followup to an article in
 the group."
   :group 'gnus-message
-  :type 'regexp)
+  :type '(choice (regexp)
+		 (const nil)))
 
 (defcustom gnus-add-to-list nil
   "*If non-nil, add a `to-list' parameter automatically."
@@ -170,9 +172,6 @@ See Info node `(gnus)Posting Styles'."
   :group 'gnus-message
   :type 'boolean)
 
-(defvar gnus-inews-mark-gcc-as-read nil
-  "Obsolete variable. Use `gnus-gcc-mark-as-read' instead.")
-
 (make-obsolete-variable 'gnus-inews-mark-gcc-as-read
 			'gnus-gcc-mark-as-read)
 
@@ -261,21 +260,21 @@ This variable is used only when `gnus-post-method' is `current'."
 
 (defcustom gnus-message-replysign
   nil
-  "Automatically sign replys to signed messages.
+  "Automatically sign replies to signed messages.
 See also the `mml-default-sign-method' variable."
   :group 'gnus-message
   :type 'boolean)
 
 (defcustom gnus-message-replyencrypt
   nil
-  "Automatically encrypt replys to encrypted messages.
+  "Automatically encrypt replies to encrypted messages.
 See also the `mml-default-encrypt-method' variable."
   :group 'gnus-message
   :type 'boolean)
 
 (defcustom gnus-message-replysignencrypted
   t
-  "Setting this causes automatically encryped messages to also be signed."
+  "Setting this causes automatically encrypted messages to also be signed."
   :group 'gnus-message
   :type 'boolean)
 
@@ -456,6 +455,7 @@ Thank you for your help in stamping out bugs.
 	 (set (make-local-variable 'gnus-newsgroup-name) ,group)
 	 (gnus-maybe-setup-default-charset)
 	 (gnus-run-hooks 'gnus-message-setup-hook))
+       (message-hide-headers)
        (gnus-add-buffer)
        (gnus-configure-windows ,config t)
        (run-hooks 'post-command-hook)
@@ -524,7 +524,7 @@ Gcc: header for archiving purposes."
 	(while (setq elem (pop alist))
 	  (when (or (and (stringp (car elem))
 			 (string-match (car elem) group))
-		    (and (gnus-functionp (car elem))
+		    (and (functionp (car elem))
 			 (funcall (car elem) group))
 		    (and (symbolp (car elem))
 			 (symbol-value (car elem))))
@@ -532,22 +532,26 @@ Gcc: header for archiving purposes."
 
 (defun gnus-inews-add-send-actions (winconf buffer article
 					    &optional config yanked)
-  (make-local-hook 'message-sent-hook)
+  (gnus-make-local-hook 'message-sent-hook)
   (add-hook 'message-sent-hook (if gnus-agent 'gnus-agent-possibly-do-gcc
 				 'gnus-inews-do-gcc) nil t)
   (when gnus-agent
-    (make-local-hook 'message-header-hook)
+    (gnus-make-local-hook 'message-header-hook)
     (add-hook 'message-header-hook 'gnus-agent-possibly-save-gcc nil t))
   (setq message-post-method
 	`(lambda (arg)
 	   (gnus-post-method arg ,gnus-newsgroup-name)))
-  (setq message-user-agent (gnus-extended-version))
-  (when (not message-use-multi-frames)
+  (setq message-user-agent (gnus-message-make-user-agent))
+  (unless message-use-multi-frames
     (message-add-action
-     `(set-window-configuration ,winconf) 'exit 'postpone 'kill))
+     `(if (gnus-buffer-exists-p ,buffer)
+	  (set-window-configuration ,winconf))
+     'exit 'postpone 'kill))
   (let ((to-be-marked (cond
-		       (yanked yanked)
-		       (article (list article))
+		       (yanked
+			(mapcar
+			 (lambda (x) (if (listp x) (car x) x)) yanked))
+		       (article (if (listp article) article (list article)))
 		       (t nil))))
     (message-add-action
      `(when (gnus-buffer-exists-p ,buffer)
@@ -598,7 +602,7 @@ If ARG is 1, prompt for group name to post to.
 
 This function prepares a news even when using mail groups.  This is useful
 for posting messages to mail groups without actually sending them over the
-network.  The corresponding backend must have a 'request-post method."
+network.  The corresponding back end must have a 'request-post method."
   (interactive "P")
   ;; We can't `let' gnus-newsgroup-name here, since that leads
   ;; to local variables leaking.
@@ -677,7 +681,7 @@ If ARG, don't do that.  If ARG is 1, prompt for group name to post to.
 
 This function prepares a news even when using mail groups.  This is useful
 for posting messages to mail groups without actually sending them over the
-network.  The corresponding backend must have a 'request-post method."
+network.  The corresponding back end must have a 'request-post method."
   (interactive "P")
   ;; We can't `let' gnus-newsgroup-name here, since that leads
   ;; to local variables leaking.
@@ -697,7 +701,12 @@ network.  The corresponding backend must have a 'request-post method."
 		  gnus-newsgroup-name))
 	  ;; #### see comment in gnus-setup-message -- drv
 	  (gnus-setup-message 'message
-	    (message-news (gnus-group-real-name gnus-newsgroup-name))))
+	    (progn
+	      (message-news (gnus-group-real-name gnus-newsgroup-name))
+	      (set (make-local-variable 'gnus-discouraged-post-methods)
+		   (delq
+		    (car (gnus-find-method-for-group gnus-newsgroup-name))
+		    (copy-sequence gnus-discouraged-post-methods))))))
       (save-excursion
 	(set-buffer buffer)
 	(setq gnus-newsgroup-name group)))))
@@ -726,8 +735,7 @@ a news."
 If prefix argument YANK is non-nil, the original article is yanked
 automatically.
 YANK is a list of elements, where the car of each element is the
-article number, and the two following numbers is the region to be
-yanked."
+article number, and the cdr is the string to be yanked."
   (interactive
    (list (and current-prefix-arg
 	      (gnus-summary-work-articles 1))))
@@ -927,7 +935,9 @@ header line with the old Message-ID."
 	      (forward-line 1))
 	    (let ((mail-header-separator ""))
 	      (setq beg (point)
-		    end (or (message-goto-body) beg)))
+		    end (or (message-goto-body)
+			    ;; There may be just a header.
+			    (point-max))))
 	    ;; Delete the headers from the displayed articles.
 	    (set-buffer gnus-article-copy)
 	    (let ((mail-header-separator ""))
@@ -1095,17 +1105,23 @@ If SILENT, don't prompt the user."
      (t gnus-select-method))))
 
 
+;; Dummies to avoid byte-compile warning.
+(eval-when-compile
+  (defvar xemacs-codename))
+
 (defun gnus-message-make-user-agent (&optional include-mime-info max-column
 						 newline-product)
   "Return a user-agent info.  If INCLUDE-MIME-INFO is non-nil and the
 variable `mime-edit-user-agent-value' is bound, the value will be
-included in the return value.  If MAX-COLUMN is specified, the return
-value will be folded up as it were filled.  NEWLINE-PRODUCT specifies
-whether a newline should be inserted in front of each product-token.
-If the value is t or `hard', it works strictly.  Otherwise, if it is
-non-nil (e.g. `soft'), it works semi-strictly.
+included in the return value, and `gnus-user-agent' is ignored.  If
+MAX-COLUMN is specified, the return value will be folded up as it were
+filled.  NEWLINE-PRODUCT specifies whether a newline should be
+inserted in front of each product-token.  If the value is t or `hard',
+it works strictly.  Otherwise, if it is non-nil (e.g. `soft'), it
+works semi-strictly.
 
-Here is an example of how to use this function:
+Here is an odd example, which inserts a User-Agent: header when you
+begin to compose a message:
 
 \(add-hook 'gnus-message-setup-hook
 	  (lambda nil
@@ -1118,12 +1134,23 @@ Here is an example of how to use this function:
 			(gnus-message-make-user-agent t 76 'soft)
 			\"\\n\")))))
 "
-  (let ((user-agent (if (and include-mime-info
-			     (boundp 'mime-edit-user-agent-value))
-			(concat (gnus-extended-version)
-				" "
-				mime-edit-user-agent-value)
-		      (gnus-extended-version))))
+  (let ((gnus-v (gnus-extended-version))
+	user-agent)
+    (cond ((and include-mime-info
+		(boundp 'mime-edit-user-agent-value))
+	   (setq user-agent (concat gnus-v " " mime-edit-user-agent-value)))
+	  ((eq gnus-user-agent 'gnus-mime-edit)
+	   (setq user-agent
+		 (if (boundp 'mime-edit-user-agent-value)
+		     (concat gnus-v " " mime-edit-user-agent-value)
+		   gnus-v)))
+	  (t
+	   (setq user-agent (if (stringp gnus-user-agent)
+				gnus-user-agent
+			      (concat gnus-v
+				      (let ((emacs-v (gnus-emacs-version)))
+					(when emacs-v
+					  (concat " " emacs-v))))))))
     (when max-column
       (unless (natnump max-column)
 	(setq max-column 76))
@@ -1199,7 +1226,7 @@ If VERY-WIDE, make a very wide reply."
 	      (gnus-summary-work-articles 1))))
   ;; Allow user to require confirmation before replying by mail to the
   ;; author of a news article (or mail message).
-  (when (or 
+  (when (or
 	    (not (or (gnus-news-group-p gnus-newsgroup-name)
 		     gnus-confirm-treat-mail-like-news))
 	    (not (cond ((stringp gnus-confirm-mail-reply-to-news)
@@ -1294,7 +1321,8 @@ automatically."
 
 (defun gnus-summary-wide-reply-with-original (n)
   "Start composing a wide reply mail to the current message.
-The original article will be yanked."
+The original article will be yanked.
+Uses the process/prefix convention."
   (interactive "P")
   (gnus-summary-reply-with-original n t))
 
@@ -1327,7 +1355,7 @@ from the original Gnus."
       (gnus-summary-digest-mail-forward nil post)
     ;; No process marks.
     (let* ((gnus-article-reply (gnus-summary-article-number))
-	   (gnus-article-yanked-articles (list (list gnus-article-reply)))
+	   (gnus-article-yanked-articles (list gnus-article-reply))
 	   charset
 	   (message-included-forward-headers
 	    (if full-headers "" message-included-forward-headers)))
@@ -1347,9 +1375,10 @@ If N is nil and any articles have been marked with the process mark,
 forward those articles instead.
 Optional POST will use news to forward instead of mail."
   (interactive "P")
-  (let ((articles (gnus-summary-work-articles n))
-	(topics "Topics:\n")
-	subject article frame)
+  (let* ((articles (gnus-summary-work-articles n))
+	 (gnus-article-yanked-articles (copy-sequence articles))
+	 (topics "Topics:\n")
+	 subject article frame)
     (when (car articles)
       (gnus-setup-message 'forward
 	(gnus-summary-select-article)
@@ -1465,8 +1494,8 @@ composing a new message."
 	(goto-char (point-max))
 	(insert mail-header-separator)
 	(goto-char (point-min))
-	(re-search-forward "^To:\\|^Newsgroups:" nil 'move)
-	(forward-char 1)
+	(when (re-search-forward "^To:\\|^Newsgroups:" nil 'move)
+	  (forward-char 1))
 	(widen)))))
 
 (defun gnus-summary-post-forward (&optional full-headers)
@@ -1562,7 +1591,7 @@ The current group name will be inserted at \"%s\".")
       ;; This mail group doesn't have a `to-list', so we add one
       ;; here.  Magic!
       (when (gnus-y-or-n-p
-	     (format "Do you want to add this as `to-list': %s " to-address))
+	     (format "Do you want to add this as `to-list': %s? " to-address))
 	(gnus-group-add-parameter group (cons 'to-list to-address))))))
 
 (defun gnus-put-message ()
@@ -1760,7 +1789,7 @@ The source file has to be in the Emacs load path."
 
 (defun gnus-summary-resend-bounced-mail (&optional fetch)
   "Re-mail the current message.
-This only makes sense if the current message is a bounce message than
+This only makes sense if the current message is a bounce message that
 contains some mail you have written which has been bounced back to
 you.
 If FETCH, try to fetch the article that this is a reply to, if indeed
@@ -1836,9 +1865,13 @@ this is a reply."
 			      group (gnus-status-message method))
 		(sit-for 2))
 	      (when (and group-art
+			 ;; FIXME: Should gcc-mark-as-read work when
+			 ;; Gnus is not running?
 			 (gnus-alive-p)
 			 (or gnus-gcc-mark-as-read
-			     gnus-inews-mark-gcc-as-read))
+			     (and
+			      (boundp 'gnus-inews-mark-gcc-as-read)
+			      (symbol-value 'gnus-inews-mark-gcc-as-read))))
 		(gnus-group-mark-article-read group (cdr group-art)))
 	      (kill-buffer (current-buffer)))))))))
 
@@ -1849,7 +1882,7 @@ this is a reply."
       (message-narrow-to-headers)
       (let* ((group gnus-outgoing-message-group)
 	     (gcc (cond
-		   ((gnus-functionp group)
+		   ((functionp group)
 		    (funcall group))
 		   ((or (stringp group) (list group))
 		    group))))
@@ -1890,7 +1923,7 @@ this is a reply."
 	   ((and (listp var) (stringp (car var)))
 	    ;; A list of groups.
 	    var)
-	   ((gnus-functionp var)
+	   ((functionp var)
 	    ;; A function.
 	    (funcall var group))
 	   (t
@@ -1903,7 +1936,7 @@ this is a reply."
 				 ;; Regexp.
 				 (when (string-match (caar var) group)
 				   (cdar var)))
-				((gnus-functionp (car var))
+				((functionp (car var))
 				 ;; Function.
 				 (funcall (car var) group))
 				(t
@@ -1927,14 +1960,17 @@ this is a reply."
 		     (if (string-match " " gcc-self-val)
 			 (concat "\"" gcc-self-val "\"")
 		       gcc-self-val)
-		   (if (string-match " " group)
-		       (concat "\"" group "\"")
-		     group)))
+		   ;; In nndoc groups, we use the parent group name
+		   ;; instead of the current group.
+		   (let ((group (or (gnus-group-find-parameter
+				     gnus-newsgroup-name 'parent-group)
+				    group)))
+		     (if (string-match " " group)
+			 (concat "\"" group "\"")
+		       group))))
 		(if (not (eq gcc-self-val 'none))
 		    (insert "\n")
-		  (progn
-		    (beginning-of-line)
-		    (kill-line))))
+		  (gnus-delete-line)))
 	    ;; Use the list of groups.
 	    (while (setq name (pop groups))
 	      (let ((str (if (string-match ":" name)
@@ -1948,6 +1984,16 @@ this is a reply."
 		(insert " ")))
 	    (insert "\n")))))))
 
+(defun gnus-mailing-list-followup-to ()
+  "Look at the headers in the current buffer and return a Mail-Followup-To address."
+  (let ((x-been-there (gnus-fetch-original-field "x-beenthere"))
+	(list-post (gnus-fetch-original-field "list-post")))
+    (when (and list-post
+	       (string-match "mailto:\\([^>]+\\)" list-post))
+      (setq list-post (match-string 1 list-post)))
+    (or list-post
+	x-been-there)))
+
 ;;; Posting styles.
 
 (defun gnus-configure-posting-styles (&optional group-name)
@@ -1980,9 +2026,9 @@ this is a reply."
 			 (and header
 			      (string-match (pop style) header))))))
 	       ((or (symbolp match)
-		    (gnus-functionp match))
+		    (functionp match))
 		(cond
-		 ((gnus-functionp match)
+		 ((functionp match)
 		  ;; Function to be called.
 		  (funcall match))
 		 ((boundp match)
@@ -2009,8 +2055,7 @@ this is a reply."
       (setq name (assq 'name results)
 	    address (assq 'address results))
       (setq results (delq name (delq address results)))
-      ;; make-local-hook is not obsolete in Emacs 20 or XEmacs.
-      (make-local-hook 'message-setup-hook)
+      (gnus-make-local-hook 'message-setup-hook)
       (setq results (sort results (lambda (x y)
 				    (string-lessp (car x) (car y)))))
       (dolist (result results)
@@ -2085,8 +2130,8 @@ this is a reply."
 	     ((stringp value)
 	      value)
 	     ((or (symbolp value)
-		  (gnus-functionp value))
-	      (cond ((gnus-functionp value)
+		  (functionp value))
+	      (cond ((functionp value)
 		     (funcall value))
 		    ((boundp value)
 		     (symbol-value value))))
