@@ -1,5 +1,5 @@
 ;;; mml2015.el --- MIME Security with Pretty Good Privacy (PGP)
-;; Copyright (C) 2000, 2001, 2002 Free Software Foundation, Inc.
+;; Copyright (C) 2000, 2001, 2002, 2003 Free Software Foundation, Inc.
 
 ;; Author: Shenghuo Zhu <zsh@cs.rochester.edu>
 ;; Keywords: PGP MIME MML
@@ -32,6 +32,7 @@
 (eval-when-compile (require 'gnus-clfns))
 (require 'mm-decode)
 (require 'mm-util)
+(require 'mml)
 
 (defvar mml2015-use (or
 		     (progn
@@ -277,8 +278,7 @@
 (defun mml2015-mailcrypt-sign (cont)
   (mc-sign-generic (message-options-get 'message-sender)
 		   nil nil nil nil)
-  (let ((boundary
-	 (funcall mml-boundary-function (incf mml-multipart-number)))
+  (let ((boundary (mml-compute-boundary cont))
 	hash point)
     (goto-char (point-min))
     (unless (re-search-forward "^-----BEGIN PGP SIGNED MESSAGE-----\r?$" nil t)
@@ -330,7 +330,7 @@
 			(or (y-or-n-p "Sign the message? ")
 			    'not))))
 	     'never)))
-    (mm-with-unibyte-current-buffer-mule4
+    (mm-with-unibyte-current-buffer
       (mc-encrypt-generic
        (or (message-options-get 'message-recipients)
 	   (message-options-set 'message-recipients
@@ -341,8 +341,7 @@
   (goto-char (point-min))
   (unless (looking-at "-----BEGIN PGP MESSAGE-----")
     (error "Fail to encrypt the message"))
-  (let ((boundary
-	 (funcall mml-boundary-function (incf mml-multipart-number))))
+  (let ((boundary (mml-compute-boundary cont)))
     (insert (format "Content-Type: multipart/encrypted; boundary=\"%s\";\n"
 		    boundary))
     (insert "\tprotocol=\"application/pgp-encrypted\"\n\n")
@@ -382,7 +381,7 @@
 	       (buffer-string)))
 	    (set-buffer cipher)
 	    (erase-buffer)
-	    (insert-buffer plain)
+	    (insert-buffer-substring plain)
 	    (goto-char (point-min))
 	    (while (search-forward "\r\n" nil t)
 	      (replace-match "\n" t t))))
@@ -544,8 +543,7 @@
      mm-security-handle 'gnus-info "Failed")))
 
 (defun mml2015-gpg-sign (cont)
-  (let ((boundary
-	 (funcall mml-boundary-function (incf mml-multipart-number)))
+  (let ((boundary (mml-compute-boundary cont))
 	(text (current-buffer)) signature)
     (goto-char (point-max))
     (unless (bolp)
@@ -572,17 +570,16 @@
       (goto-char (point-max))
       (insert (format "\n--%s\n" boundary))
       (insert "Content-Type: application/pgp-signature\n\n")
-      (insert-buffer signature)
+      (insert-buffer-substring signature)
       (goto-char (point-max))
       (insert (format "--%s--\n" boundary))
       (goto-char (point-max)))))
 
 (defun mml2015-gpg-encrypt (cont &optional sign)
-  (let ((boundary
-	 (funcall mml-boundary-function (incf mml-multipart-number)))
+  (let ((boundary (mml-compute-boundary cont))
 	(text (current-buffer))
 	cipher)
-    (mm-with-unibyte-current-buffer-mule4
+    (mm-with-unibyte-current-buffer
       (with-temp-buffer
 	;; set up a function to call the correct gpg encrypt routine
 	;; with the right arguments. (FIXME: this should be done
@@ -626,7 +623,7 @@
 	(insert "Version: 1\n\n")
 	(insert (format "--%s\n" boundary))
 	(insert "Content-Type: application/octet-stream\n\n")
-	(insert-buffer cipher)
+	(insert-buffer-substring cipher)
 	(goto-char (point-max))
 	(insert (format "--%s--\n" boundary))
 	(goto-char (point-max))))))
@@ -703,7 +700,7 @@
 	     (buffer-string))))
 	(progn
 	  (erase-buffer)
-	  (insert-buffer pgg-output-buffer)
+	  (insert-buffer-substring pgg-output-buffer)
 	  (goto-char (point-min))
 	  (while (search-forward "\r\n" nil t)
 	    (replace-match "\n" t t))
@@ -774,11 +771,12 @@
 
 (defun mml2015-pgg-clear-verify ()
   (let ((pgg-errors-buffer mml2015-result-buffer)
-	(text (current-buffer)))
+	(text (buffer-string))
+	(coding-system buffer-file-coding-system))
     (if (condition-case err
 	    (prog1
 		(mm-with-unibyte-buffer
-		  (insert-buffer text)
+		  (insert (encode-coding-string text coding-system))
 		  (pgg-verify-region (point-min) (point-max) nil t))
 	      (goto-char (point-min))
 	      (while (search-forward "\r\n" nil t)
@@ -806,7 +804,9 @@
 
 (defun mml2015-pgg-sign (cont)
   (let ((pgg-errors-buffer mml2015-result-buffer)
-	(boundary (funcall mml-boundary-function (incf mml-multipart-number))))
+	(boundary (mml-compute-boundary cont))
+	(pgg-default-user-id (or (message-options-get 'mml-sender)
+				 pgg-default-user-id)))
     (unless (pgg-sign-region (point-min) (point-max))
       (pop-to-buffer mml2015-result-buffer)
       (error "Sign error"))
@@ -819,14 +819,14 @@
     (goto-char (point-max))
     (insert (format "\n--%s\n" boundary))
     (insert "Content-Type: application/pgp-signature\n\n")
-    (insert-buffer pgg-output-buffer)
+    (insert-buffer-substring pgg-output-buffer)
     (goto-char (point-max))
     (insert (format "--%s--\n" boundary))
     (goto-char (point-max))))
 
 (defun mml2015-pgg-encrypt (cont &optional sign)
   (let ((pgg-errors-buffer mml2015-result-buffer)
-	(boundary (funcall mml-boundary-function (incf mml-multipart-number))))
+	(boundary (mml-compute-boundary cont)))
     (unless (pgg-encrypt-region (point-min) (point-max)
 				(split-string
 				 (or
@@ -847,7 +847,7 @@
     (insert "Version: 1\n\n")
     (insert (format "--%s\n" boundary))
     (insert "Content-Type: application/octet-stream\n\n")
-    (insert-buffer pgg-output-buffer)
+    (insert-buffer-substring pgg-output-buffer)
     (goto-char (point-max))
     (insert (format "--%s--\n" boundary))
     (goto-char (point-max))))
