@@ -1,7 +1,7 @@
 ;;; nnheader.el --- header access macros for Semi-gnus and its backends
 
 ;; Copyright (C) 1987, 1988, 1989, 1990, 1993, 1994, 1995, 1996,
-;;        1997, 1998, 2000, 2001, 2002
+;;        1997, 1998, 2000, 2001, 2002, 2003
 ;;        Free Software Foundation, Inc.
 
 ;; Author: Masanobu UMEDA <umerin@flab.flab.fujitsu.junet>
@@ -36,9 +36,10 @@
 
 ;; Requiring `gnus-util' at compile time creates a circular
 ;; dependency between nnheader.el and gnus-util.el.
-;(eval-when-compile (require 'gnus-util))
+;;(eval-when-compile (require 'gnus-util))
 
 (require 'mail-utils)
+(require 'gnus-util)
 
 ;; Reduce the required value of `recursive-load-depth-limit' for Emacs 21.
 (require 'pces)
@@ -79,6 +80,14 @@ Integer values will in effect be rounded up to the nearest multiple of
 (defvar nnheader-head-chop-length 2048
   "*Length of each read operation when trying to fetch HEAD headers.")
 
+(defvar nnheader-read-timeout
+  (if (string-match "windows-nt\\|os/2\\|emx\\|cygwin"
+		    (symbol-name system-type))
+      1.0				; why?
+    0.1)
+  "How long nntp should wait between checking for the end of output.
+Shorter values mean quicker response, but are more CPU intensive.")
+
 (defvar nnheader-file-name-translation-alist
   (let ((case-fold-search t))
     (cond
@@ -120,6 +129,10 @@ This variable is a substitute for `mm-text-coding-system-for-write'.")
    (t nil))
   "Coding system of auto save file.")
 
+(defvar nnheader-directory-separator-character
+  (string-to-char (substring (file-name-as-directory ".") -1))
+  "*A character used to a directory separator.")
+
 (eval-and-compile
   (autoload 'nnmail-message-id "nnmail")
   (autoload 'mail-position-on-field "sendmail")
@@ -288,20 +301,29 @@ nil, ."
 		       temporary-file-directory))))
 	  (if dir-flag
 	      (make-directory file))
-	  file)))))
+	  file))))
+
+  ;; Should keep track of `mm-coding-system-p' in mm-util.el.
+  (defun nnheader-coding-system-p (sym)
+    "Return non-nil if SYM is a coding system."
+    (or (and (fboundp 'find-coding-system) (find-coding-system sym))
+	(and (fboundp 'coding-system-p) (coding-system-p sym))))
+  (defalias 'mm-coding-system-p 'nnheader-coding-system-p))
 
 ;; mail-parse stuff.
 (unless (featurep 'mail-parse)
-  ;; Should keep track of `rfc2047-narrow-to-field' in rfc2047.el.
-  (defun-maybe std11-narrow-to-field ()
-    "Narrow the buffer to the header on the current line."
-    (forward-line 0)
-    (narrow-to-region (point)
-		      (progn
-			(std11-field-end)
-			(when (eolp) (forward-line 1))
-			(point)))
-    (goto-char (point-min)))
+  (unless (fboundp 'std11-narrow-to-field)
+    (defalias 'std11-narrow-to-field
+      ;; Should keep track of `rfc2047-narrow-to-field' in rfc2047.el.
+      (lambda ()
+	"Narrow the buffer to the header on the current line."
+	(forward-line 0)
+	(narrow-to-region (point)
+			  (progn
+			    (std11-field-end)
+			    (when (eolp) (forward-line 1))
+			    (point)))
+	(goto-char (point-min)))))
   (defalias 'mail-header-narrow-to-field 'std11-narrow-to-field)
 
   ;; Should keep track of `ietf-drums-narrow-to-header' in ietf-drums.el.
@@ -314,128 +336,136 @@ nil, ."
        (point-max)))
     (goto-char (point-min)))
 
-  ;; Should keep track of `rfc2047-fold-region' in rfc2047.el.
-  (defun-maybe std11-fold-region (b e)
-    "Fold long lines in region B to E."
-    (save-restriction
-      (narrow-to-region b e)
-      (goto-char (point-min))
-      (let ((break nil)
-	    (qword-break nil)
-	    (first t)
-	    (bol (save-restriction
-		   (widen)
-		   (gnus-point-at-bol))))
-	(while (not (eobp))
-	  (when (and (or break qword-break)
-		     (> (- (point) bol) 76))
-	    (goto-char (or break qword-break))
-	    (setq break nil
-		  qword-break nil)
-	    (if (looking-at "[ \t]")
-		(insert "\n")
-	      (insert "\n "))
-	    (setq bol (1- (point)))
-	    ;; Don't break before the first non-LWSP characters.
-	    (skip-chars-forward " \t")
-	    (unless (eobp)
-	      (forward-char 1)))
-	  (cond
-	   ((eq (char-after) ?\n)
-	    (forward-char 1)
-	    (setq bol (point)
-		  break nil
-		  qword-break nil)
-	    (skip-chars-forward " \t")
-	    (unless (or (eobp) (eq (char-after) ?\n))
-	      (forward-char 1)))
-	   ((eq (char-after) ?\r)
-	    (forward-char 1))
-	   ((memq (char-after) '(?  ?\t))
-	    (skip-chars-forward " \t")
-	    (if first
-		;; Don't break just after the header name.
-		(setq first nil)
-	      (setq break (1- (point)))))
-	   ((not break)
-	    (if (not (looking-at "=\\?[^=]"))
-		(if (eq (char-after) ?=)
-		    (forward-char 1)
-		  (skip-chars-forward "^ \t\n\r="))
-	      (setq qword-break (point))
-	      (skip-chars-forward "^ \t\n\r")))
-	   (t
-	    (skip-chars-forward "^ \t\n\r"))))
-	(when (and (or break qword-break)
-		   (> (- (point) bol) 76))
-	  (goto-char (or break qword-break))
-	  (setq break nil
-		qword-break nil)
-	  (if (looking-at "[ \t]")
-	      (insert "\n")
-	    (insert "\n "))
-	  (setq bol (1- (point)))
-	  ;; Don't break before the first non-LWSP characters.
-	  (skip-chars-forward " \t")
-	  (unless (eobp)
-	    (forward-char 1))))))
-
-  ;; Should keep track of `rfc2047-fold-field' in rfc2047.el.
-  (defun-maybe std11-fold-field ()
-    "Fold the current line."
-    (save-excursion
-      (save-restriction
-	(std11-narrow-to-field)
-	(std11-fold-region (point-min) (point-max)))))
+  (unless (fboundp 'std11-fold-region)
+    (defalias 'std11-fold-region
+      ;; Should keep track of `rfc2047-fold-region' in rfc2047.el.
+      (lambda (b e)
+	"Fold long lines in region B to E."
+	(save-restriction
+	  (narrow-to-region b e)
+	  (goto-char (point-min))
+	  (let ((break nil)
+		(qword-break nil)
+		(first t)
+		(bol (save-restriction
+		       (widen)
+		       (gnus-point-at-bol))))
+	    (while (not (eobp))
+	      (when (and (or break qword-break)
+			 (> (- (point) bol) 76))
+		(goto-char (or break qword-break))
+		(setq break nil
+		      qword-break nil)
+		(if (looking-at "[ \t]")
+		    (insert "\n")
+		  (insert "\n "))
+		(setq bol (1- (point)))
+		;; Don't break before the first non-LWSP characters.
+		(skip-chars-forward " \t")
+		(unless (eobp)
+		  (forward-char 1)))
+	      (cond
+	       ((eq (char-after) ?\n)
+		(forward-char 1)
+		(setq bol (point)
+		      break nil
+		      qword-break nil)
+		(skip-chars-forward " \t")
+		(unless (or (eobp) (eq (char-after) ?\n))
+		  (forward-char 1)))
+	       ((eq (char-after) ?\r)
+		(forward-char 1))
+	       ((memq (char-after) '(?  ?\t))
+		(skip-chars-forward " \t")
+		(if first
+		    ;; Don't break just after the header name.
+		    (setq first nil)
+		  (setq break (1- (point)))))
+	       ((not break)
+		(if (not (looking-at "=\\?[^=]"))
+		    (if (eq (char-after) ?=)
+			(forward-char 1)
+		      (skip-chars-forward "^ \t\n\r="))
+		  (setq qword-break (point))
+		  (skip-chars-forward "^ \t\n\r")))
+	       (t
+		(skip-chars-forward "^ \t\n\r"))))
+	    (when (and (or break qword-break)
+		       (> (- (point) bol) 76))
+	      (goto-char (or break qword-break))
+	      (setq break nil
+		    qword-break nil)
+	      (if (looking-at "[ \t]")
+		  (insert "\n")
+		(insert "\n "))
+	      (setq bol (1- (point)))
+	      ;; Don't break before the first non-LWSP characters.
+	      (skip-chars-forward " \t")
+	      (unless (eobp)
+		(forward-char 1))))))))
 
+  (unless (fboundp 'std11-fold-field)
+    (defalias 'std11-fold-field
+      ;; Should keep track of `rfc2047-fold-field' in rfc2047.el.
+      (lambda ()
+	"Fold the current line."
+	(save-excursion
+	  (save-restriction
+	    (std11-narrow-to-field)
+	    (std11-fold-region (point-min) (point-max)))))))
   (defalias 'mail-header-fold-field 'std11-fold-field)
 
-  ;; Should keep track of `rfc2047-unfold-region' in rfc2047.el.
-  (defun-maybe std11-unfold-region (b e)
-    "Unfold lines in region B to E."
-    (save-restriction
-      (narrow-to-region b e)
-      (goto-char (point-min))
-      (let ((bol (save-restriction
-		   (widen)
-		   (gnus-point-at-bol)))
-	    (eol (gnus-point-at-eol)))
-	(forward-line 1)
-	(while (not (eobp))
-	  (if (and (looking-at "[ \t]")
-		   (< (- (gnus-point-at-eol) bol) 76))
-	      (delete-region eol (progn
-				   (goto-char eol)
-				   (skip-chars-forward "\r\n")
-				   (point)))
-	    (setq bol (gnus-point-at-bol)))
-	  (setq eol (gnus-point-at-eol))
-	  (forward-line 1)))))
-
-  ;; Should keep track of `rfc2047-unfold-field' in rfc2047.el.
-  (defun-maybe std11-unfold-field ()
-    "Fold the current line."
-    (save-excursion
-      (save-restriction
-	(std11-narrow-to-field)
-	(std11-unfold-region (point-min) (point-max)))))
+  (unless (fboundp 'std11-unfold-region)
+    (defalias 'std11-unfold-region
+      ;; Should keep track of `rfc2047-unfold-region' in rfc2047.el.
+      (lambda (b e)
+	"Unfold lines in region B to E."
+	(save-restriction
+	  (narrow-to-region b e)
+	  (goto-char (point-min))
+	  (let ((bol (save-restriction
+		       (widen)
+		       (gnus-point-at-bol)))
+		(eol (gnus-point-at-eol)))
+	    (forward-line 1)
+	    (while (not (eobp))
+	      (if (and (looking-at "[ \t]")
+		       (< (- (gnus-point-at-eol) bol) 76))
+		  (delete-region eol (progn
+				       (goto-char eol)
+				       (skip-chars-forward "\r\n")
+				       (point)))
+		(setq bol (gnus-point-at-bol)))
+	      (setq eol (gnus-point-at-eol))
+	      (forward-line 1)))))))
 
+  (unless (fboundp 'std11-unfold-field)
+    (defalias 'std11-unfold-field
+      ;; Should keep track of `rfc2047-unfold-field' in rfc2047.el.
+      (lambda ()
+	"Fold the current line."
+	(save-excursion
+	  (save-restriction
+	    (std11-narrow-to-field)
+	    (std11-unfold-region (point-min) (point-max)))))))
   (defalias 'mail-header-unfold-field 'std11-unfold-field)
 
-  ;; This is the original function in T-gnus.
-  (defun-maybe std11-extract-addresses-components (string)
-    "Extract a list of full name and canonical address from STRING.  Each
+  (unless (fboundp 'std11-extract-addresses-components)
+    (defalias 'std11-extract-addresses-components
+      ;; This is the original function in T-gnus.
+      (lambda (string)
+	"Extract a list of full name and canonical address from STRING.  Each
 element looks like a list of the form (FULL-NAME CANONICAL-ADDRESS).
 If no name can be extracted, FULL-NAME will be nil."
-    (when string
-      (let (addresses)
-	(dolist (structure (std11-parse-addresses-string
-			    (std11-unfold-string string))
-			   addresses)
-	  (push (list (std11-full-name-string structure)
-		      (std11-address-string structure))
-		addresses))
-	(nreverse addresses))))
+	(when string
+	  (let (addresses)
+	    (dolist (structure (std11-parse-addresses-string
+				(std11-unfold-string string))
+			       addresses)
+	      (push (list (std11-full-name-string structure)
+			  (std11-address-string structure))
+		    addresses))
+	    (nreverse addresses))))))
 
   ;; Should keep track of `ietf-drums-parse-addresses' in ietf-drums.el.
   (defun mail-header-parse-addresses (string)
@@ -466,7 +496,6 @@ given, the return value will not contain the last newline."
 	  (setq value (buffer-substring start (point)))))
       (goto-char begin)
       value))
-
   (defalias 'mail-header-field-value 'std11-field-value))
 
 ;; ietf-drums stuff.
@@ -876,7 +905,7 @@ the line could be found."
 	(prev (point-min))
 	num found)
     (while (not found)
-      (goto-char (/ (+ max min) 2))
+      (goto-char (+ min (/ (- max min) 2)))
       (beginning-of-line)
       (if (or (= (point) prev)
 	      (eobp))
@@ -884,8 +913,7 @@ the line could be found."
 	(setq prev (point))
 	(while (and (not (numberp (setq num (read cur))))
 		    (not (eobp)))
-	  (delete-region (progn (beginning-of-line) (point))
-			 (progn (forward-line 1) (point))))
+	  (gnus-delete-line))
 	(cond ((> num article)
 	       (setq max (point)))
 	      ((< num article)
@@ -1172,7 +1200,7 @@ list of headers that match SEQUENCE (see `nntp-retrieve-headers')."
       ;; This is invalid, but not all articles have Message-IDs.
       ()
     (mail-position-on-field "References")
-    (let ((begin (save-excursion (beginning-of-line) (point)))
+    (let ((begin (gnus-point-at-bol))
 	  (fill-column 78)
 	  (fill-prefix "\t"))
       (when references
@@ -1234,7 +1262,7 @@ list of headers that match SEQUENCE (see `nntp-retrieve-headers')."
   "Regexp that matches numerical file names.")
 
 (defvar nnheader-numerical-full-files (concat "/" nnheader-numerical-files)
-  "Regexp that matches numerical full file paths.")
+  "Regexp that matches numerical full file names.")
 
 (defsubst nnheader-file-to-number (file)
   "Take a FILE name and return the article number."
@@ -1299,7 +1327,8 @@ If FULL, translate everything."
 	;; We translate -- but only the file name.  We leave the directory
 	;; alone.
 	(if (and (featurep 'xemacs)
-		 (memq system-type '(cygwin32 win32 w32 mswindows windows-nt)))
+		 (memq system-type '(cygwin32 win32 w32 mswindows windows-nt
+					      cygwin)))
 	    ;; This is needed on NT and stuff, because
 	    ;; file-name-nondirectory is not enough to split
 	    ;; file names, containing ':', e.g.
@@ -1397,7 +1426,7 @@ without formatting."
 		     (expand-file-name
 		      (file-name-as-directory top))))
        (error "")))
-   ?/ ?.))
+   nnheader-directory-separator-character ?.))
 
 (defun nnheader-message (level &rest args)
   "Message if the Gnus backends are talkative."
@@ -1412,10 +1441,10 @@ without formatting."
       (<= level gnus-verbose-backends)))
 
 (defvar nnheader-pathname-coding-system 'binary
-  "*Coding system for pathname.")
+  "*Coding system for file name.")
 
 (defun nnheader-group-pathname (group dir &optional file)
-  "Make pathname for GROUP."
+  "Make file name for GROUP."
   (concat
    (let ((dir (file-name-as-directory (expand-file-name dir))))
      ;; If this directory exists, we use it directly.
@@ -1431,11 +1460,6 @@ without formatting."
 	 ((numberp file) (int-to-string file))
 	 (t file))))
 
-(defun nnheader-functionp (form)
-  "Return non-nil if FORM is funcallable."
-  (or (and (symbolp form) (fboundp form))
-      (and (listp form) (eq (car form) 'lambda))))
-
 (defun nnheader-concat (dir &rest files)
   "Concat DIR as directory to FILES."
   (apply 'concat (file-name-as-directory dir) files))
@@ -1449,11 +1473,15 @@ without formatting."
   "Return the file size of FILE or 0."
   (or (nth 7 (file-attributes file)) 0))
 
-(defun nnheader-find-etc-directory (package &optional file)
-  "Go through the path and find the \".../etc/PACKAGE\" directory.
-If FILE, find the \".../etc/PACKAGE\" file instead."
+(defun nnheader-find-etc-directory (package &optional file first)
+  "Go through `load-path' and find the \"../etc/PACKAGE\" directory.
+This function will look in the parent directory of each `load-path'
+entry, and look for the \"etc\" directory there.
+If FILE, find the \".../etc/PACKAGE\" file instead.
+If FIRST is non-nil, return the directory or the file found at the
+first.  Otherwise, find the newest one, though it may take a time."
   (let ((path load-path)
-	dir result)
+	dir results)
     ;; We try to find the dir by looking at the load path,
     ;; stripping away the last component and adding "etc/".
     (while path
@@ -1465,10 +1493,14 @@ If FILE, find the \".../etc/PACKAGE\" file instead."
 			   "etc/" package
 			   (if file "" "/"))))
 	       (or file (file-directory-p dir)))
-	  (setq result dir
-		path nil)
+	  (progn
+	    (or (member dir results)
+		(push dir results))
+	    (setq path (if first nil (cdr path))))
 	(setq path (cdr path))))
-    result))
+    (if (or first (not (cdr results)))
+	(car results)
+      (car (sort results 'file-newer-than-file-p)))))
 
 (eval-when-compile
   (defvar ange-ftp-path-format)
@@ -1504,21 +1536,22 @@ find-file-hooks, etc.
 (defun nnheader-insert-nov-file (file first)
   (let ((size (nth 7 (file-attributes file)))
 	(cutoff (* 32 1024)))
-    (if (< size cutoff)
-	;; If the file is small, we just load it.
-	(nnheader-insert-file-contents file)
-      ;; We start on the assumption that FIRST is pretty recent.  If
-      ;; not, we just insert the rest of the file as well.
-      (let (current)
-	(nnheader-insert-file-contents file nil (- size cutoff) size)
-	(goto-char (point-min))
-	(delete-region (point) (or (search-forward "\n" nil 'move) (point)))
-	(setq current (ignore-errors (read (current-buffer))))
-	(if (and (numberp current)
-		 (< current first))
-	    t
-	  (delete-region (point-min) (point-max))
-	  (nnheader-insert-file-contents file))))))
+    (when size
+      (if (< size cutoff)
+          ;; If the file is small, we just load it.
+          (nnheader-insert-file-contents file)
+        ;; We start on the assumption that FIRST is pretty recent.  If
+        ;; not, we just insert the rest of the file as well.
+        (let (current)
+          (nnheader-insert-file-contents file nil (- size cutoff) size)
+          (goto-char (point-min))
+          (delete-region (point) (or (search-forward "\n" nil 'move) (point)))
+          (setq current (ignore-errors (read (current-buffer))))
+          (if (and (numberp current)
+                   (< current first))
+              t
+            (delete-region (point-min) (point-max))
+            (nnheader-insert-file-contents file)))))))
 
 (defun nnheader-find-file-noselect (&rest args)
   (let ((format-alist nil)
@@ -1622,6 +1655,14 @@ find-file-hooks, etc.
 	standard-output
       (call-process shell-file-name nil t nil shell-command-switch command))))
 
+(defun nnheader-accept-process-output (process)
+  (accept-process-output
+   process
+   (truncate nnheader-read-timeout)
+   (truncate (* (- nnheader-read-timeout
+		   (truncate nnheader-read-timeout))
+		1000))))
+
 (when (featurep 'xemacs)
   (require 'nnheaderxm))
 
