@@ -1,6 +1,6 @@
 ;;; nnmh.el --- mhspool access for Gnus
 
-;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001
+;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2003
 ;;	Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
@@ -239,7 +239,7 @@ as unread by Gnus.")
 	  (goto-char (point-max))
 	  (insert
 	   (format
-	    "%s %d %d y\n"
+	    "%s %.0f %.0f y\n"
 	    (progn
 	      (string-match
 	       (regexp-quote
@@ -297,8 +297,8 @@ as unread by Gnus.")
 (deffoo nnmh-close-group (group &optional server)
   t)
 
-(deffoo nnmh-request-move-article
-    (article group server accept-form &optional last)
+(deffoo nnmh-request-move-article (article group server
+					   accept-form &optional last)
   (let ((buf (get-buffer-create " *nnmh move*"))
 	result)
     (and
@@ -325,7 +325,10 @@ as unread by Gnus.")
 	   (not (equal group "draft")))
       (nnmail-check-syntax))
   (when nnmail-cache-accepted-message-ids
-    (nnmail-cache-insert (nnmail-fetch-field "message-id") group))
+    (nnmail-cache-insert (nnmail-fetch-field "message-id") 
+			 group
+			 (nnmail-fetch-field "subject")
+			 (nnmail-fetch-field "from")))
   (nnheader-init-server-buffer)
   (prog1
       (if (stringp group)
