@@ -1,6 +1,6 @@
 ;;; pop3.el --- Post Office Protocol (RFC 1460) interface
 
-;; Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002
+;; Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003
 ;;        Free Software Foundation, Inc.
 
 ;; Author: Richard L. Pieri <ratinox@peorth.gweep.net>
@@ -45,7 +45,10 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
+(eval-when-compile
+  (require 'cl)
+  ;; For compiling this module in Gnus with XEmacs -no-autoloads.
+  (require 'advice))
 
 ;; as-binary-process, open-network-stream-as-binary, write-region-as-binary
 (require 'pces)
@@ -89,9 +92,9 @@ Used for APOP authentication.")
 (defvar pop3-uidl-file-name "~/.uidls"
   "File in which to store the UIDL of processed messages.")
 
-(defvar pop3-uidl-support 'dont-know
-  "Whether the server supports UIDL.
-Nil means no, t means yes, not-nil-or-t means yet to be determined.")
+(defvar pop3-uidl-support nil
+  "Alist of servers and flags of whether they support UIDLs.
+Users don't have to set this value.")
 
 (defvar pop3-uidl-obarray (make-vector 31 0)
   "Uidl hash table.")
@@ -133,7 +136,7 @@ Nil means no, t means yes, not-nil-or-t means yet to be determined.")
     ;; query for password
     (if (and pop3-password-required (not pop3-password))
 	(setq pop3-password
-	      (pop3-read-passwd (format "Password for %s: " pop3-maildrop))))
+	      (read-passwd (format "Password for %s: " pop3-maildrop))))
     (cond ((equal 'apop pop3-authentication-scheme)
 	   (pop3-apop process pop3-maildrop))
 	  ((equal 'pass pop3-authentication-scheme)
@@ -185,7 +188,7 @@ Nil means no, t means yes, not-nil-or-t means yet to be determined.")
     ;; query for password
     (if (and pop3-password-required (not pop3-password))
 	(setq pop3-password
-	      (pop3-read-passwd (format "Password for %s: " pop3-maildrop))))
+	      (read-passwd (format "Password for %s: " pop3-maildrop))))
     (cond ((equal 'apop pop3-authentication-scheme)
 	   (pop3-apop process pop3-maildrop))
 	  ((equal 'pass pop3-authentication-scheme)
@@ -222,6 +225,9 @@ Argument PORT specifies connecting port."
 			 (+ 1 (or (string-match ">" response) -1)))))
       process)))
 
+(eval-when-compile
+  (autoload 'open-ssl-stream "ssl"))
+
 (defun pop3-open-ssl-stream-1 (name buffer host service extra-arg)
   (require 'ssl)
   (let* ((ssl-program-name
@@ -238,7 +244,7 @@ Argument PORT specifies connecting port."
 		    (goto-char (point-max))
 		    (forward-line -1)
 		    (not (looking-at "+OK")))
-	  (accept-process-output process 1)
+	  (nnheader-accept-process-output process)
 	  (sit-for 1))
 	(delete-region (point-min) (point)))
       (and process (memq (process-status process) '(open run))
@@ -290,8 +296,9 @@ Return the response string if optional second argument RETURN is non-nil."
     (save-excursion
       (set-buffer (process-buffer process))
       (goto-char pop3-read-point)
-      (while (not (search-forward "\r\n" nil t))
-	(accept-process-output process 0 500)
+      (while (and (memq (process-status process) '(open run))
+		  (not (search-forward "\r\n" nil t)))
+	(nnheader-accept-process-output process)
 	(goto-char pop3-read-point))
       (setq match-end (point))
       (goto-char pop3-read-point)
@@ -305,17 +312,6 @@ Return the response string if optional second argument RETURN is non-nil."
 	    t)
 	  )))))
 
-(defvar pop3-read-passwd nil)
-(defun pop3-read-passwd (prompt)
-  (if (not pop3-read-passwd)
-      (if (fboundp 'read-passwd)
-	  (setq pop3-read-passwd 'read-passwd)
-	(if (load "passwd" t)
-	    (setq pop3-read-passwd 'read-passwd)
-	  (autoload 'ange-ftp-read-passwd "ange-ftp")
-	  (setq pop3-read-passwd 'ange-ftp-read-passwd))))
-  (funcall pop3-read-passwd prompt))
-
 (defun pop3-clean-region (start end)
   (setq end (set-marker (make-marker) end))
   (save-excursion
@@ -425,7 +421,8 @@ If NOW, use that time instead."
       ;; only retrieve messages matching our regexp or in the uidl list
       (when (and
 	     ;; remove elements not in the uidl, this assumes the uidl is short
-	     (or (not (eq pop3-uidl-support t))
+	     (or (not (and pop3-leave-mail-on-server
+			   (cdr (assoc pop3-mailhost pop3-uidl-support))))
 		 (memq (caar messages) uidl))
 	     (caar messages)
 	     ;; don't download messages that are too large
@@ -436,14 +433,18 @@ If NOW, use that time instead."
 				     (pop3-top process (caar messages) 0)))))
 	(push (car messages) out))
       (setq messages (cdr messages)))
-    (cons total (reverse out))))
+    (cons total (nreverse out))))
 
 (defun pop3-get-uidl (process)
   "Use PROCESS to get a list of unread message numbers."
-  (let ((messages (pop3-uidl process)) uidl)
-    (if (or (null messages) (null pop3-uidl-support))
-	(setq pop3-uidl-support nil)
-      (setq pop3-uidl-support t)
+  (let ((messages (pop3-uidl process))
+	(support (assoc pop3-mailhost pop3-uidl-support))
+	uidl)
+    (if support
+	(setcdr support (and messages t))
+      (push (cons pop3-mailhost (and messages t))
+	    pop3-uidl-support))
+    (when messages
       (save-excursion
 	(with-temp-buffer
 	  (when (file-readable-p pop3-uidl-file-name)
@@ -452,15 +453,13 @@ If NOW, use that time instead."
 	  (while (looking-at "\\([^ \n\t]+\\)")
 	    (set (intern (match-string 1) pop3-uidl-obarray)
 		 (cons nil t))
-	    (forward-line 1))
-	  ))
+	    (forward-line 1))))
       (dolist (message (cdr messages))
 	(if (setq uidl (intern-soft (cdr message) pop3-uidl-obarray))
 	    (setcar (symbol-value uidl) (car message))
 	  (set (intern (cdr message) pop3-uidl-obarray)
 	       (cons (car message) nil))))
-      (pop3-get-unread-message-numbers))
-    ))
+      (pop3-get-unread-message-numbers))))
 
 (defun pop3-get-unread-message-numbers ()
   "Return a sorted list of unread msg numbers to retrieve."
@@ -516,18 +515,16 @@ If NOW, use that time instead."
 ;; tree, `md5' might have been defined in w3/md5.el, ./lpath.el or one
 ;; of some other libraries and `md5' will accept only 3 arguments.  We
 ;; will deceive the byte-compiler not to say warnings.
-(eval-and-compile
-  (if (fboundp 'eval-when)
-      ;; `eval-when' might not be provided when loading .el file.
-      (eval-when 'compile
-	(let ((def (assq 'md5 byte-compile-function-environment)))
-	  (if def
-	      (setcdr def '(lambda (object &optional start end
-					   coding-system noerror)))
-	    (setq byte-compile-function-environment
-		  (cons '(md5 . (lambda (object &optional start end
-						coding-system noerror)))
-			byte-compile-function-environment)))))))
+(eval-when-compile
+  (if (boundp 'byte-compile-function-environment)
+      (let ((def (assq 'md5 byte-compile-function-environment)))
+	(if def
+	    (setcdr def '(lambda (object &optional start end
+					 coding-system noerror)))
+	  (setq byte-compile-function-environment
+		(cons '(md5 . (lambda (object &optional start end
+					      coding-system noerror)))
+		      byte-compile-function-environment))))))
 
 ;; Note that `pop3-md5' should never encode a given string to use for
 ;; the apop authentication.
@@ -558,7 +555,7 @@ Check whether the 4th argument CODING-SYSTEM is allowed"
   (let ((pass pop3-password))
     (if (and pop3-password-required (not pass))
 	(setq pass
-	      (pop3-read-passwd (format "Password for %s: " pop3-maildrop))))
+	      (read-passwd (format "Password for %s: " pop3-maildrop))))
     (if pass
 	(let ((hash (pop3-md5 (concat pop3-timestamp pass))))
 	  (pop3-send-command process (format "APOP %s %s" user hash))
@@ -710,7 +707,8 @@ If msgno is invalid, return nil.  Otherwise, return a string."
     (set-buffer (process-buffer process))
     (goto-char start)
     (while (not (re-search-forward "^\\.\r\n" nil t))
-      (accept-process-output process 3)
+      ;; Fixme: Shouldn't depend on nnheader.
+      (nnheader-accept-process-output process)
       (goto-char start))
     (setq pop3-read-point (point-marker))
     (goto-char (match-beginning 0))
